@@ -19,7 +19,7 @@
  * add two std::vector without copying them (0 copy).
  */
 
-#include <cassert>
+#include <assert.h>
 #include <vector>
 
 #ifdef PRINT_OUTPUT
@@ -66,8 +66,9 @@ int main(int argc, char **argv)
 
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
-	conf.nmic = 0;
+	/* starpu_data_get_user_data cannot work in master-slave */
 	conf.nmpi_ms = 0;
+	conf.ntcpip_ms = 0;
 
 	// initialize StarPU with default configuration
 	auto ret = starpu_init(&conf);
