@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,7 +26,7 @@ int main(int argc, char **argv)
 	return 77;
 }
 #else
-#include <cassert>
+#include <assert.h>
 #include <vector>
 
 #ifdef PRINT_OUTPUT
@@ -61,9 +61,9 @@ class my_allocator
 		node = a.get_node();
 	}
 
-	explicit my_allocator(const unsigned node)
+	explicit my_allocator(const unsigned thenode)
 	{
-		this->node = node;
+		this->node = thenode;
 	}
 
 	pointer allocate(size_type n, const void * = 0)
@@ -148,8 +148,6 @@ struct vector_cpp_interface
 	enum starpu_data_interface_id id;
 
 	uintptr_t ptr;
-	uintptr_t dev_handle;
-	size_t offset;
 	uint32_t nx;
 	size_t elemsize;
 	std::vector<MY_TYPE>* vec;
@@ -165,96 +163,86 @@ static int vector_interface_copy_any_to_any(void *src_interface, unsigned src_no
 #if __cplusplus >= 201103L
 static const struct starpu_data_copy_methods vector_cpp_copy_data_methods_s =
 {
-
 	.can_copy = NULL,
 
 	.ram_to_ram = NULL,
 	.ram_to_cuda = NULL,
+	.ram_to_hip = NULL,
 	.ram_to_opencl = NULL,
-	.ram_to_mic = NULL,
+	.ram_to_max_fpga = NULL,
 
 	.cuda_to_ram = NULL,
 	.cuda_to_cuda = NULL,
-	.cuda_to_opencl = NULL,
+
+	.hip_to_ram = NULL,
+	.hip_to_hip = NULL,
 
 	.opencl_to_ram = NULL,
-	.opencl_to_cuda = NULL,
 	.opencl_to_opencl = NULL,
 
-	.mic_to_ram = NULL,
-
-	.ram_to_mpi_ms = NULL,
-	.mpi_ms_to_ram = NULL,
-	.mpi_ms_to_mpi_ms = NULL,
+	.max_fpga_to_ram = NULL,
 
 	.ram_to_cuda_async = NULL,
 	.cuda_to_ram_async = NULL,
 	.cuda_to_cuda_async = NULL,
 
+	.ram_to_hip_async = NULL,
+	.hip_to_ram_async = NULL,
+	.hip_to_hip_async = NULL,
+
 	.ram_to_opencl_async = NULL,
 	.opencl_to_ram_async = NULL,
 	.opencl_to_opencl_async = NULL,
 
-	.ram_to_mpi_ms_async = NULL,
-	.mpi_ms_to_ram_async = NULL,
-	.mpi_ms_to_mpi_ms_async = NULL,
-
-	.ram_to_mic_async = NULL,
-	.mic_to_ram_async = NULL,
+	.ram_to_max_fpga_async = NULL,
+	.max_fpga_to_ram_async = NULL,
 
 	.any_to_any = vector_interface_copy_any_to_any,
 };
 #else
 static const struct starpu_data_copy_methods vector_cpp_copy_data_methods_s =
 {
-	NULL,
+ 	NULL,                      // can_copy
 
-	NULL,
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // ram_to_ram
+	NULL,                      // ram_to_cuda
+	NULL,                      // ram_to_hip
+	NULL,                      // ram_to_opencl
+	NULL,                      // ram_to_max_fpga
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // cuda_to_ram
+	NULL,                      // cuda_to_cuda
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // hip_to_ram
+	NULL,                      // hip_to_hip
 
-	NULL,
+	NULL,                      // opencl_to_ram
+	NULL,                      // opencl_to_opencl
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // max_fpga_to_ram
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // ram_to_cuda_async
+	NULL,                      // cuda_to_ram_async
+	NULL,                      // cuda_to_cuda_async
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // ram_to_hip_async
+	NULL,                      // hip_to_ram_async
+	NULL,                      // hip_to_hip_async
 
-	NULL,
-	NULL,
-	NULL,
+	NULL,                      // ram_to_opencl_async
+	NULL,                      // opencl_to_ram_async
+	NULL,                      // opencl_to_opencl_async
 
-	NULL,
-	NULL,
-	NULL,
-
-	NULL,
-	NULL,
+	NULL,                      // ram_to_max_fpga_asyn
+	NULL,                      // max_fpga_to_ram_asyn
 
 	vector_interface_copy_any_to_any,
 };
 #endif
 
-static void register_vector_cpp_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_vector_cpp_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static starpu_ssize_t allocate_vector_cpp_buffer_on_node(void *data_interface_, unsigned dst_node);
 static void *vector_cpp_to_pointer(void *data_interface, unsigned node);
-static int vector_cpp_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_vector_cpp_buffer_on_node(void *data_interface, unsigned node);
 static void free_vector_cpp_buffer_on_node(void *data_interface, unsigned node);
 static size_t vector_cpp_interface_get_size(starpu_data_handle_t handle);
@@ -262,6 +250,7 @@ static uint32_t footprint_vector_cpp_interface_crc32(starpu_data_handle_t handle
 static int vector_cpp_compare(void *data_interface_a, void *data_interface_b);
 static void display_vector_cpp_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t vector_cpp_describe(void *data_interface, char *buf, size_t size);
 
@@ -272,13 +261,18 @@ static struct starpu_data_interface_ops interface_vector_cpp_ops =
 	.unregister_data_handle = NULL,
 	.allocate_data_on_node = allocate_vector_cpp_buffer_on_node,
 	.free_data_on_node = free_vector_cpp_buffer_on_node,
+	.cache_data_on_node = NULL,
+	.reuse_data_on_node = NULL,
+	.map_data = NULL,
+	.unmap_data = NULL,
+	.update_map = NULL,
 	.init = NULL,
 	.copy_methods = &vector_cpp_copy_data_methods_s,
 	.handle_to_pointer = NULL,
 	.to_pointer = vector_cpp_to_pointer,
-	.pointer_is_inside = vector_cpp_pointer_is_inside,
 	.get_size = vector_cpp_interface_get_size,
 	.get_alloc_size = NULL,
+	.get_max_size = NULL,
 	.footprint = footprint_vector_cpp_interface_crc32,
 	.alloc_footprint = NULL,
 	.compare = vector_cpp_compare,
@@ -291,7 +285,11 @@ static struct starpu_data_interface_ops interface_vector_cpp_ops =
 	.dontcache = 0,
 	.get_mf_ops = NULL,
 	.pack_data = pack_vector_cpp_handle,
+	.peek_data = peek_vector_cpp_handle,
 	.unpack_data = unpack_vector_cpp_handle,
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL,
 	.name = (char *) "VECTOR_CPP_INTERFACE"
 };
 #else
@@ -302,11 +300,15 @@ static struct starpu_data_interface_ops interface_vector_cpp_ops =
 	allocate_vector_cpp_buffer_on_node,
 	free_vector_cpp_buffer_on_node,
 	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 	&vector_cpp_copy_data_methods_s,
 	vector_cpp_to_pointer,
-	vector_cpp_pointer_is_inside,
 	vector_cpp_interface_get_size,
 	NULL,
+	NULL,
 	footprint_vector_cpp_interface_crc32,
 	NULL,
 	vector_cpp_compare,
@@ -319,7 +321,11 @@ static struct starpu_data_interface_ops interface_vector_cpp_ops =
 	0,
 	NULL,
 	pack_vector_cpp_handle,
+	peek_vector_cpp_handle,
 	unpack_vector_cpp_handle,
+	NULL,
+	NULL,
+	NULL,
 	(char *) "VECTOR_CPP_INTERFACE"
 };
 #endif
@@ -332,20 +338,11 @@ static void *vector_cpp_to_pointer(void *data_interface, unsigned node)
 	return (void*) vector_interface->ptr;
 }
 
-static int vector_cpp_pointer_is_inside(void *data_interface, unsigned int node, void *ptr)
-{
-	(void) node;
-	struct vector_cpp_interface *vector_interface = (struct vector_cpp_interface *) data_interface;
-
-	return (char*) ptr >= (char*) vector_interface->ptr &&
-		(char*) ptr < (char*) vector_interface->ptr + vector_interface->nx*vector_interface->elemsize;
-}
-
-static void register_vector_cpp_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_vector_cpp_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct vector_cpp_interface *vector_interface = (struct vector_cpp_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct vector_cpp_interface *local_interface = (struct vector_cpp_interface *)
@@ -354,15 +351,11 @@ static void register_vector_cpp_handle(starpu_data_handle_t handle, unsigned hom
 		if (node == home_node)
 		{
 			local_interface->ptr = vector_interface->ptr;
-                        local_interface->dev_handle = vector_interface->dev_handle;
-                        local_interface->offset = vector_interface->offset;
 			local_interface->vec = vector_interface->vec;
 		}
 		else
 		{
 			local_interface->ptr = 0;
-                        local_interface->dev_handle = 0;
-                        local_interface->offset = 0;
 			local_interface->vec = NULL;
 		}
 
@@ -382,8 +375,6 @@ void vector_cpp_data_register(starpu_data_handle_t *handleptr, int home_node,
 	{
 		.id = STARPU_UNKNOWN_INTERFACE_ID,
 		.ptr = (uintptr_t) &(*vec)[0],
-                .dev_handle = (uintptr_t) &(*vec)[0],
-                .offset = 0,
 		.nx = nx,
 		.elemsize = elemsize,
 		.vec = vec,
@@ -403,11 +394,6 @@ void vector_cpp_data_register(starpu_data_handle_t *handleptr, int home_node,
 	};
 #endif
 
-	if (interface_vector_cpp_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
-	{
-		interface_vector_cpp_ops.interfaceid = (enum starpu_data_interface_id )starpu_data_interface_get_next_id();
-	}
-
 	starpu_data_register(handleptr, home_node, &vector, &interface_vector_cpp_ops);
 }
 
@@ -462,7 +448,7 @@ static int pack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, vo
 	return 0;
 }
 
-static int unpack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -472,6 +458,13 @@ static int unpack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node,
 	STARPU_ASSERT(count == vector_interface->elemsize * vector_interface->nx);
 	memcpy((void*)vector_interface->ptr, ptr, count);
 
+	return 0;
+}
+
+static int unpack_vector_cpp_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_vector_cpp_handle(handle, node, ptr, count);
+
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 
 	return 0;
@@ -519,8 +512,6 @@ static starpu_ssize_t allocate_vector_cpp_buffer_on_node(void *data_interface_,
 
 	/* update the data properly in consequence */
 	vector_interface->ptr = (uintptr_t) &((*vec)[0]);
-	vector_interface->dev_handle = (uintptr_t) &((*vec)[0]);
-        vector_interface->offset = 0;
 
 	return allocated_memory;
 }
@@ -530,6 +521,9 @@ static void free_vector_cpp_buffer_on_node(void *data_interface, unsigned node)
 	struct vector_cpp_interface *vector_interface = (struct vector_cpp_interface *) data_interface;
 
 	delete vector_interface->vec;
+
+	vector_interface->vec = NULL;
+	vector_interface->ptr = 0;
 }
 
 static int vector_interface_copy_any_to_any(void *src_interface, unsigned src_node,
@@ -539,8 +533,8 @@ static int vector_interface_copy_any_to_any(void *src_interface, unsigned src_no
 	struct vector_cpp_interface *dst_vector = (struct vector_cpp_interface *) dst_interface;
 	int ret;
 
-	ret = starpu_interface_copy(src_vector->dev_handle, src_vector->offset, src_node,
-				    dst_vector->dev_handle, dst_vector->offset, dst_node,
+	ret = starpu_interface_copy(src_vector->ptr, 0, src_node,
+				    dst_vector->ptr, 0, dst_node,
 				    src_vector->nx*src_vector->elemsize, async_data);
 
 	return ret;
@@ -586,6 +580,7 @@ int main(int argc, char **argv)
 	starpu_conf_init(&conf);
 	/* _starpu_src_common_execute_kernel doesn't support this yet */
 	conf.nmpi_ms = 0;
+	conf.ntcpip_ms = 0;
 
 	// initialize StarPU with default configuration
 	int ret = starpu_init(&conf);
