@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011,2013,2015,2017                 CNRS
- * Copyright (C) 2011,2015                                Université de Bordeaux
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,24 +18,24 @@
 
 #include <starpu.h>
 
-void cpu_func(void *buffers[], void *cl_arg)
+void block_cpu_func(void *buffers[], void *cl_arg)
 {
-        int i, j, k;
-        int *factor = (int *) cl_arg;
+	int i, j, k;
+	int *factor = (int *) cl_arg;
 	int *block = (int *)STARPU_BLOCK_GET_PTR(buffers[0]);
 	int nx = (int)STARPU_BLOCK_GET_NX(buffers[0]);
 	int ny = (int)STARPU_BLOCK_GET_NY(buffers[0]);
 	int nz = (int)STARPU_BLOCK_GET_NZ(buffers[0]);
-        unsigned ldy = STARPU_BLOCK_GET_LDY(buffers[0]);
-        unsigned ldz = STARPU_BLOCK_GET_LDZ(buffers[0]);
+	unsigned ldy = STARPU_BLOCK_GET_LDY(buffers[0]);
+	unsigned ldz = STARPU_BLOCK_GET_LDZ(buffers[0]);
 
-        for(k=0; k<nz ; k++)
+	for(k=0; k<nz ; k++)
 	{
-                for(j=0; j<ny ; j++)
+		for(j=0; j<ny ; j++)
 		{
-                        for(i=0; i<nx ; i++)
-                                block[(k*ldz)+(j*ldy)+i] = *factor;
-                }
-        }
+			for(i=0; i<nx ; i++)
+				block[(k*ldz)+(j*ldy)+i] *= *factor;
+		}
+	}
 }
 
