@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -38,8 +38,7 @@ static unsigned use_cg = 0; /* use a LU decomposition of CG ? */
 static unsigned no_prio = 0;
 
 extern void do_conjugate_gradient(float *nzvalA, float *vecb, float *vecx, uint32_t nnz,
-              		unsigned nrow, uint32_t *colind, uint32_t *rowptr);
-
+				  unsigned nrow, uint32_t *colind, uint32_t *rowptr);
 
 static void parse_args(int argc, char **argv)
 {
@@ -330,12 +329,12 @@ static float compute_A_value(unsigned i, unsigned j, point *pmesh)
 	theta_j = NODE_TO_THETA(j);
 
 	/* Compute the Sum of all the integral over all triangles */
-	if ( (abs((int)thick_i - (int)thick_j) <= 1) && (abs((int)theta_i - (int)theta_j) <= 1) )
+	if ((abs((int)thick_i - (int)thick_j) <= 1) && (abs((int)theta_i - (int)theta_j) <= 1))
 	{
-		if ( (theta_j == theta_i -1) && (thick_j == thick_i +1))
+		if ((theta_j == theta_i -1) && (thick_j == thick_i +1))
 			goto done;
 
-		if ( (theta_j == theta_i + 1) && (thick_j == thick_i  - 1))
+		if ((theta_j == theta_i + 1) && (thick_j == thick_i  - 1))
 			goto done;
 
 		/* this may not be a null entry */
@@ -355,8 +354,8 @@ static void solve_system(unsigned size, unsigned subsize, float *result, int *Re
 	unsigned i;
 
 	/* solve the actual problem LU X = B */
-        /* solve LX' = Y with X' = UX */
-        /* solve UX = X' */
+	/* solve LX' = Y with X' = UX */
+	/* solve UX = X' */
 	FPRINTF(stderr, "Solving the problem ...\n");
 
 	float *savedB = NULL;
@@ -524,11 +523,11 @@ static unsigned long build_neighbour_vector(unsigned long*neighbours, unsigned n
 	int dtheta, dthick;
 	for (dthick = -1; dthick <= 1; dthick++)
 	{
-		if ((former_thick + dthick) >= 0 && (former_thick + dthick) <= (int)nthick )
+		if ((former_thick + dthick) >= 0 && (former_thick + dthick) <= (int)nthick)
 		{
 			for (dtheta = -1; dtheta <= 1; dtheta++)
 			{
-				if ((former_theta + dtheta) >= 0 && (former_theta + dtheta) <= (int)ntheta )
+				if ((former_theta + dtheta) >= 0 && (former_theta + dtheta) <= (int)ntheta)
 				{
 					/* we got a possible neighbour */
 					unsigned pnode =
@@ -648,7 +647,9 @@ static unsigned build_sparse_stiffness_matrix_A(point *pmesh, float **nzval, uin
 				if (val != 0.0f)
 				{
 					*nzval = realloc(*nzval, (pos+1)*sizeof(float));
+					STARPU_ASSERT(*nzval);
 					*colind = realloc(*colind, (pos+1)*sizeof(uint32_t));
+					STARPU_ASSERT(*colind);
 
 					(*nzval)[pos] = val;
 					(*colind)[pos] = nodeneighbour;
@@ -799,7 +800,7 @@ int main(int argc, char **argv)
 
 		starpu_cublas_shutdown();
 		starpu_shutdown();
-		free_system(A, B, pinned);
+		free_system(A, B, newsize, pinned);
 	}
 
 #ifdef STARPU_OPENGL_RENDER
