@@ -1,7 +1,6 @@
 ! StarPU --- Runtime system for heterogeneous multicore architectures.
 !
-! Copyright (C) 2017                                     CNRS
-! Copyright (C) 2016                                     Inria
+! Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 !
 ! StarPU is free software; you can redistribute it and/or modify
 ! it under the terms of the GNU Lesser General Public License as published by
@@ -64,7 +63,7 @@ program nf_varbuf
                 do i=0,nbuffers-1
                         call fstarpu_data_descr_array_set(descrs_var, i, dh_var, FSTARPU_RW)
                 end do
-                call fstarpu_insert_task((/ cl_varbuf, &
+                call fstarpu_task_insert((/ cl_varbuf, &
                         FSTARPU_VALUE, c_loc(nbuffers), FSTARPU_SZ_C_INT, &
                         FSTARPU_DATA_MODE_ARRAY, descrs_var, c_loc(nbuffers), &
                         C_NULL_PTR /))
