@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -59,7 +59,7 @@ static int can_execute(unsigned workerid, struct starpu_task *task, unsigned nim
 	(void)task;
 	(void)nimpl;
 	enum starpu_worker_archtype type = starpu_worker_get_type(workerid);
-	if (type == STARPU_CPU_WORKER || type == STARPU_OPENCL_WORKER || type == STARPU_MIC_WORKER)
+	if (type == STARPU_CPU_WORKER || type == STARPU_OPENCL_WORKER)
 		return 1;
 
 #ifdef STARPU_USE_CUDA
@@ -103,7 +103,7 @@ void init_cuda_func(void *descr[], void *cl_arg)
 void init_opencl_func(void *buffers[], void *cl_arg)
 {
 	(void)cl_arg;
-        cl_int err;
+	cl_int err;
 	cl_command_queue queue;
 
 	cl_mem dot = (cl_mem) STARPU_VARIABLE_GET_PTR(buffers[0]);
@@ -111,17 +111,16 @@ void init_opencl_func(void *buffers[], void *cl_arg)
 	DOT_TYPE zero = (DOT_TYPE) 0.0;
 
 	err = clEnqueueWriteBuffer(queue,
-			dot,
-			CL_TRUE,
-			0,
-			sizeof(DOT_TYPE),
-			&zero,
-			0,
-			NULL,
-			NULL);
+				   dot,
+				   CL_TRUE,
+				   0,
+				   sizeof(DOT_TYPE),
+				   &zero,
+				   0,
+				   NULL,
+				   NULL);
 	if (err != CL_SUCCESS)
 		STARPU_OPENCL_REPORT_ERROR(err);
-
 }
 #endif
 
@@ -164,7 +163,7 @@ void redux_opencl_func(void *buffers[], void *args)
 {
 	(void)args;
 	int id, devid;
-        cl_int err;
+	cl_int err;
 	cl_kernel kernel;
 	cl_command_queue queue;
 
@@ -175,25 +174,21 @@ void redux_opencl_func(void *buffers[], void *args)
 	devid = starpu_worker_get_devid(id);
 
 	err = starpu_opencl_load_kernel(&kernel, &queue, &_opencl_program, "_redux_opencl", devid);
-	if (err != CL_SUCCESS)
-		STARPU_OPENCL_REPORT_ERROR(err);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
 	err = clSetKernelArg(kernel, 0, sizeof(dota), &dota);
 	err|= clSetKernelArg(kernel, 1, sizeof(dotb), &dotb);
-	if (err != CL_SUCCESS)
-		STARPU_OPENCL_REPORT_ERROR(err);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
 	{
 		size_t global=1;
-                size_t local=1;
-                size_t s;
-                cl_device_id device;
+		size_t local=1;
+		cl_device_id device;
 
-                starpu_opencl_get_device(devid, &device);
+		starpu_opencl_get_device(devid, &device);
 
 		err = clEnqueueNDRangeKernel(queue, kernel, 1, NULL, &global, &local, 0, NULL, NULL);
-		if (err != CL_SUCCESS)
-			STARPU_OPENCL_REPORT_ERROR(err);
+		if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 	}
 	starpu_opencl_release_kernel(kernel);
 }
@@ -212,7 +207,7 @@ static struct starpu_codelet redux_codelet =
 	.opencl_funcs = {redux_opencl_func},
 	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
-	.modes = {STARPU_RW, STARPU_R},
+	.modes = {STARPU_RW|STARPU_COMMUTE, STARPU_R},
 	.nbuffers = 2,
 	.name = "redux"
 };
@@ -275,7 +270,7 @@ void dot_opencl_func(void *buffers[], void *cl_arg)
 {
 	(void)cl_arg;
 	int id, devid;
-        cl_int err;
+	cl_int err;
 	cl_kernel kernel;
 	cl_command_queue queue;
 
@@ -288,27 +283,23 @@ void dot_opencl_func(void *buffers[], void *cl_arg)
 	devid = starpu_worker_get_devid(id);
 
 	err = starpu_opencl_load_kernel(&kernel, &queue, &_opencl_program, "_dot_opencl", devid);
-	if (err != CL_SUCCESS)
-		STARPU_OPENCL_REPORT_ERROR(err);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
 	err = clSetKernelArg(kernel, 0, sizeof(x), &x);
 	err|= clSetKernelArg(kernel, 1, sizeof(y), &y);
 	err|= clSetKernelArg(kernel, 2, sizeof(dot), &dot);
 	err|= clSetKernelArg(kernel, 3, sizeof(n), &n);
-	if (err != CL_SUCCESS)
-		STARPU_OPENCL_REPORT_ERROR(err);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
 	{
 		size_t global=1;
-                size_t local=1;
-                size_t s;
-                cl_device_id device;
+		size_t local=1;
+		cl_device_id device;
 
-                starpu_opencl_get_device(devid, &device);
+		starpu_opencl_get_device(devid, &device);
 
 		err = clEnqueueNDRangeKernel(queue, kernel, 1, NULL, &global, &local, 0, NULL, NULL);
-		if (err != CL_SUCCESS)
-			STARPU_OPENCL_REPORT_ERROR(err);
+		if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 	}
 	starpu_opencl_release_kernel(kernel);
 }
@@ -340,7 +331,7 @@ int main(void)
 	int ret;
 
 	/* Not supported yet */
-	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
+	if (starpu_getenv_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
 		return 77;
 
 	ret = starpu_init(NULL);
@@ -374,15 +365,15 @@ int main(void)
 	unsigned long nelems = _nblocks*_entries_per_block;
 	size_t size = nelems*sizeof(float);
 
-	_x = (float *) malloc(size);
-	_y = (float *) malloc(size);
+	starpu_malloc((void **)&_x, size*sizeof(float));
+	starpu_malloc((void **)&_y, size*sizeof(float));
 
 	_x_handles = (starpu_data_handle_t *) calloc(_nblocks, sizeof(starpu_data_handle_t));
 	_y_handles = (starpu_data_handle_t *) calloc(_nblocks, sizeof(starpu_data_handle_t));
 
 	assert(_x && _y);
 
-        starpu_srand48(0);
+	starpu_srand48(0);
 
 	DOT_TYPE reference_dot = 0.0;
 
@@ -442,13 +433,14 @@ int main(void)
 #endif
 
 #ifdef STARPU_USE_OPENCL
-        ret = starpu_opencl_unload_opencl(&_opencl_program);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
+	ret = starpu_opencl_unload_opencl(&_opencl_program);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
 #endif
 	starpu_shutdown();
 
-	free(_x);
-	free(_y);
+
+	starpu_free_noflag(_x, size*sizeof(float));
+	starpu_free_noflag(_y, size*sizeof(float));
 	free(_x_handles);
 	free(_y_handles);
 
@@ -461,8 +453,9 @@ int main(void)
 	}
 
 enodev:
+	starpu_shutdown();
 	FPRINTF(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
- 	 * could perform the kernel, so this is not an error from StarPU */
+	 * could perform the kernel, so this is not an error from StarPU */
 	return 77;
 }
