@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -69,10 +69,10 @@ void create_data(void)
 /*	printf("size = %d \n ", size); */
 
 	/* initiate the 2 vectors */
-	vector_in_ptr = malloc(size*sizeof(float));
+	starpu_malloc((void **)&vector_in_ptr, size*sizeof(float));
 	assert(vector_in_ptr);
 
-	vector_out_ptr = malloc(size*sizeof(float));
+	starpu_malloc((void **)&vector_out_ptr, size*sizeof(float));
 	assert(vector_out_ptr);
 
 	/* fill those */
@@ -109,7 +109,7 @@ void init_problem_callback(void *arg)
 /*	if (val < 10)
 		printf("callback %d remaining \n", val); */
 
-	if ( val == 0 )
+	if (val == 0)
 	{
 		ANNOTATE_HAPPENS_AFTER(&remaining);
 		printf("DONE ...\n");
@@ -119,11 +119,6 @@ void init_problem_callback(void *arg)
 	}
 }
 
-unsigned get_bcsr_nchildren(struct starpu_data_filter *f, starpu_data_handle_t handle)
-{
-  return (unsigned)starpu_bcsr_get_nnz(handle);
-}
-
 void call_filters(void)
 {
 
@@ -131,7 +126,7 @@ void call_filters(void)
 	struct starpu_data_filter vector_in_f, vector_out_f;
 
 	bcsr_f.filter_func    = starpu_bcsr_filter_canonical_block;
-	bcsr_f.get_nchildren = get_bcsr_nchildren;
+	bcsr_f.get_nchildren = starpu_bcsr_filter_canonical_block_get_nchildren;
 	/* the children use a matrix interface ! */
 	bcsr_f.get_child_ops = starpu_bcsr_filter_canonical_block_child_ops;
 
@@ -229,7 +224,7 @@ void launch_spmv_codelets(void)
 				task->handles[1] = starpu_data_get_sub_data(vector_in, 1, i);
 				task->handles[2] = starpu_data_get_sub_data(vector_out, 1, j);
 
-				/* all tasks in the same row are dependant so that we don't wait too much for data
+				/* all tasks in the same row are dependent so that we don't wait too much for data
 				 * we need to wait on the previous task if we are not the first task of a row */
 				if (index != rowptr[row & ~0x3])
 				{
@@ -336,7 +331,12 @@ int main(int argc, char *argv[])
 	FPRINTF(stderr, "Computation took (in ms)\n");
 	FPRINTF(stdout, "%2.2f\n", timing/1000);
 	FPRINTF(stderr, "Flop %e\n", totalflop);
-	FPRINTF(stderr, "GFlops : %2.2f\n", totalflop/timing/1000);
+	FPRINTF(stderr, "GFlop/s : %2.2f\n", totalflop/timing/1000);
+
+	starpu_free_noflag(vector_in_ptr, size*sizeof(float));
+	starpu_free_noflag(vector_out_ptr, size*sizeof(float));
+
+	starpu_shutdown();
 
 	return 0;
 }
