@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2015-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -45,7 +45,7 @@ void starpu_my_data_compare_codelet_cpu(void *descr[], void *_args)
 	*compare = (d0 == d1 && c0 == c1);
 }
 
-void _starpu_my_data_datatype_allocate(MPI_Datatype *mpi_datatype)
+void _starpu_my_data_datatype_allocate(unsigned node, MPI_Datatype *mpi_datatype)
 {
 	int ret;
 	int blocklengths[2] = {1, 1};
@@ -68,19 +68,20 @@ void _starpu_my_data_datatype_allocate(MPI_Datatype *mpi_datatype)
 	free(myinterface);
 }
 
-int starpu_my_data_datatype_allocate(starpu_data_handle_t handle, MPI_Datatype *mpi_datatype)
+int starpu_my_data_datatype_allocate(starpu_data_handle_t handle, unsigned node, MPI_Datatype *mpi_datatype)
 {
 	(void)handle;
-	_starpu_my_data_datatype_allocate(mpi_datatype);
+	_starpu_my_data_datatype_allocate(node, mpi_datatype);
 	return 0;
 }
 
 void starpu_my_data_datatype_free(MPI_Datatype *mpi_datatype)
 {
-	MPI_Type_free(mpi_datatype);
+	int ret = MPI_Type_free(mpi_datatype);
+	STARPU_ASSERT_MSG(ret == MPI_SUCCESS, "MPI_Type_free failed");
 }
 
-int starpu_my_data2_datatype_allocate(starpu_data_handle_t handle, MPI_Datatype *mpi_datatype)
+int starpu_my_data2_datatype_allocate(starpu_data_handle_t handle, unsigned node, MPI_Datatype *mpi_datatype)
 {
 	(void)handle;
 	(void)mpi_datatype;
@@ -120,11 +121,11 @@ char starpu_my_data_get_char(starpu_data_handle_t handle)
 	return data->c;
 }
 
-static void data_register_data_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void data_register_data_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_my_data_interface *my_data_interface = (struct starpu_my_data_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_my_data_interface *local_interface =
@@ -133,14 +134,14 @@ static void data_register_data_handle(starpu_data_handle_t handle, unsigned home
 		if (node == home_node)
 		{
 			local_interface->ptr = my_data_interface->ptr;
-                        local_interface->dev_handle = my_data_interface->dev_handle;
-                        local_interface->offset = my_data_interface->offset;
+			local_interface->dev_handle = my_data_interface->dev_handle;
+			local_interface->offset = my_data_interface->offset;
 		}
 		else
 		{
 			local_interface->ptr = 0;
-                        local_interface->dev_handle = 0;
-                        local_interface->offset = 0;
+			local_interface->dev_handle = 0;
+			local_interface->offset = 0;
 		}
 	}
 }
@@ -162,7 +163,7 @@ static starpu_ssize_t data_allocate_data_on_node(void *data_interface, unsigned
 	/* update the data properly in consequence */
 	my_data_interface->ptr = addr;
 	my_data_interface->dev_handle = handle;
-        my_data_interface->offset = 0;
+	my_data_interface->offset = 0;
 
 	return allocated_memory;
 }
@@ -171,6 +172,8 @@ static void data_free_data_on_node(void *data_interface, unsigned node)
 {
 	struct starpu_my_data_interface *my_data_interface = (struct starpu_my_data_interface *) data_interface;
 	starpu_free_on_node(node, my_data_interface->dev_handle, sizeof(int)+sizeof(char));
+	my_data_interface->ptr = 0;
+	my_data_interface->dev_handle = 0;
 }
 
 static size_t data_get_size(starpu_data_handle_t handle)
@@ -201,6 +204,15 @@ static int data_pack_data(starpu_data_handle_t handle, unsigned node, void **ptr
 	return 0;
 }
 
+static int data_peek_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	(void)handle;
+	(void)node;
+	(void)ptr;
+	STARPU_ASSERT_MSG(0, "The data interface has been registered with starpu_mpi_datatype_register(). Calling the unpack_data function should not happen\n");
+	return 0;
+}
+
 static int data_unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	(void)handle;
@@ -231,7 +243,7 @@ static int data_pack_data2(starpu_data_handle_t handle, unsigned node, void **pt
 	return 0;
 }
 
-static int data_unpack_data2(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int data_peek_data2(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	(void)count;
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
@@ -243,6 +255,12 @@ static int data_unpack_data2(starpu_data_handle_t handle, unsigned node, void *p
 	char *x = ptr;
 	x += sizeof(int);
 	memcpy(&data->c, x, sizeof(char));
+	return 0;
+}
+
+static int data_unpack_data2(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	data_peek_data2(handle, node, ptr, count);
 
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 	return 0;
@@ -301,6 +319,7 @@ static struct starpu_data_interface_ops interface_data_ops =
 	.interface_size = sizeof(struct starpu_my_data_interface),
 	.to_pointer = data_to_pointer,
 	.pack_data = data_pack_data,
+	.peek_data = data_peek_data,
 	.unpack_data = data_unpack_data,
 	.describe = data_describe
 };
@@ -310,12 +329,12 @@ void starpu_my_data_register(starpu_data_handle_t *handleptr, unsigned home_node
 	if (interface_data_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
 	{
 		interface_data_ops.interfaceid = starpu_data_interface_get_next_id();
-		starpu_mpi_interface_datatype_register(interface_data_ops.interfaceid, starpu_my_data_datatype_allocate, starpu_my_data_datatype_free);
+		starpu_mpi_interface_datatype_node_register(interface_data_ops.interfaceid, starpu_my_data_datatype_allocate, starpu_my_data_datatype_free);
 	}
 
 	struct starpu_my_data_interface data =
 	{
-	 	.id = interface_data_ops.interfaceid,
+		.id = interface_data_ops.interfaceid,
 		.ptr = (uintptr_t) xc,
 		.dev_handle = (uintptr_t) xc,
 		.offset = 0,
@@ -343,6 +362,7 @@ static struct starpu_data_interface_ops interface_data2_ops =
 	.interface_size = sizeof(struct starpu_my_data_interface),
 	.to_pointer = data_to_pointer,
 	.pack_data = data_pack_data2,
+	.peek_data = data_peek_data2,
 	.unpack_data = data_unpack_data2,
 	.describe = data_describe
 };
@@ -352,12 +372,12 @@ void starpu_my_data2_register(starpu_data_handle_t *handleptr, unsigned home_nod
 	if (interface_data2_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
 	{
 		interface_data2_ops.interfaceid = starpu_data_interface_get_next_id();
-		starpu_mpi_interface_datatype_register(interface_data2_ops.interfaceid, starpu_my_data2_datatype_allocate, starpu_my_data2_datatype_free);
+		starpu_mpi_interface_datatype_node_register(interface_data2_ops.interfaceid, starpu_my_data2_datatype_allocate, starpu_my_data2_datatype_free);
 	}
 
 	struct starpu_my_data_interface data =
 	{
-	 	.id = interface_data_ops.interfaceid,
+		.id = interface_data_ops.interfaceid,
 		.ptr = (uintptr_t) xc,
 		.dev_handle = (uintptr_t) xc,
 		.offset = 0,
