@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2021       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,9 +21,11 @@
 #include <starpu.h>
 #include <starpu_data.h>
 #include <common/utils.h>
+#include <common/uthash.h>
 #include <util/starpu_task_insert_utils.h>
 #include <datawizard/coherency.h>
 #include <core/task.h>
+#include <core/workers.h>
 
 #include <starpu_mpi_private.h>
 #include <starpu_mpi_cache.h>
@@ -32,14 +35,30 @@
 
 #define _SEND_DATA(data, mode, dest, data_tag, prio, comm, callback, arg)     \
 	do {									\
-		if (mode & STARPU_SSEND)					\
-			starpu_mpi_issend_detached_prio(data, dest, data_tag, prio, comm, callback, arg); 	\
+		if (mode & STARPU_SSEND)				\
+			return starpu_mpi_issend_detached_prio(data, dest, data_tag, prio, comm, callback, arg); \
 		else												\
-			starpu_mpi_isend_detached_prio(data, dest, data_tag, prio, comm, callback, arg);	\
+			return starpu_mpi_isend_detached_prio(data, dest, data_tag, prio, comm, callback, arg);	\
 	} while (0)
 
 static void (*pre_submit_hook)(struct starpu_task *task) = NULL;
 
+/* reduction wrap-up */
+// entry in the table
+struct _starpu_redux_data_entry
+{
+	UT_hash_handle hh;
+	starpu_data_handle_t data_handle;
+};
+// the table
+static struct _starpu_redux_data_entry *_redux_data = NULL;
+
+void _starpu_mpi_pre_submit_hook_call(struct starpu_task *task)
+{
+	if (pre_submit_hook)
+		pre_submit_hook(task);
+}
+
 int starpu_mpi_pre_submit_hook_register(void (*f)(struct starpu_task *))
 {
 	if (pre_submit_hook)
@@ -94,13 +113,13 @@ int _starpu_mpi_find_executee_node(starpu_data_handle_t data, enum starpu_data_a
 	return 0;
 }
 
-void _starpu_mpi_exchange_data_before_execution(starpu_data_handle_t data, enum starpu_data_access_mode mode, int me, int xrank, int do_execute, int prio, MPI_Comm comm)
+int _starpu_mpi_exchange_data_before_execution(starpu_data_handle_t data, enum starpu_data_access_mode mode, int me, int xrank, int do_execute, int prio, MPI_Comm comm)
 {
 	if (data && xrank == STARPU_MPI_PER_NODE)
 	{
 		STARPU_ASSERT_MSG(starpu_mpi_data_get_rank(data) == STARPU_MPI_PER_NODE, "If task is replicated, it has to access only per-node data");
 	}
-	if (data && mode & STARPU_R)
+	if (data && mode & STARPU_R && !(mode & STARPU_MPI_REDUX))
 	{
 		int mpi_rank = starpu_mpi_data_get_rank(data);
 		starpu_mpi_tag_t data_tag = starpu_mpi_data_get_tag(data);
@@ -117,8 +136,10 @@ void _starpu_mpi_exchange_data_before_execution(starpu_data_handle_t data, enum
 			{
 				if (data_tag == -1)
 					_STARPU_ERROR("StarPU needs to be told the MPI tag of this data, using starpu_mpi_data_register\n");
-				_STARPU_MPI_DEBUG(1, "Receiving data %p from %d\n", data, mpi_rank);
-				starpu_mpi_irecv_detached(data, mpi_rank, data_tag, comm, NULL, NULL);
+				_STARPU_MPI_DEBUG(1, "Receiving data %p from %d with prio %d\n", data, mpi_rank, prio);
+				int ret = starpu_mpi_irecv_detached_prio(data, mpi_rank, data_tag, prio, comm, NULL, NULL);
+				if (ret)
+					return ret;
 			}
 			// else the node has already received the data
 		}
@@ -131,25 +152,28 @@ void _starpu_mpi_exchange_data_before_execution(starpu_data_handle_t data, enum
 			{
 				if (data_tag == -1)
 					_STARPU_ERROR("StarPU needs to be told the MPI tag of this data, using starpu_mpi_data_register\n");
-				_STARPU_MPI_DEBUG(1, "Sending data %p to %d\n", data, xrank);
+				_STARPU_MPI_DEBUG(1, "Sending data %p to %d with prio %d\n", data, xrank, prio);
 				_SEND_DATA(data, mode, xrank, data_tag, prio, comm, NULL, NULL);
 			}
 			// Else the data has already been sent
 		}
 	}
+	return 0;
 }
 
 static
-void _starpu_mpi_exchange_data_after_execution(starpu_data_handle_t data, enum starpu_data_access_mode mode, int me, int xrank, int do_execute, int prio, MPI_Comm comm)
+int _starpu_mpi_exchange_data_after_execution(starpu_data_handle_t data, enum starpu_data_access_mode mode, int me, int xrank, int do_execute, int prio, MPI_Comm comm)
 {
-	if (mode & STARPU_W)
+	if (mode & STARPU_W && !(mode & STARPU_MPI_REDUX))
 	{
 		int mpi_rank = starpu_mpi_data_get_rank(data);
 		starpu_mpi_tag_t data_tag = starpu_mpi_data_get_tag(data);
+		struct _starpu_mpi_data* mpi_data = _starpu_mpi_data_get(data);
 		if(mpi_rank == -1)
 		{
 			_STARPU_ERROR("StarPU needs to be told the MPI rank of this data, using starpu_mpi_data_register\n");
 		}
+		mpi_data->modified=1;
 		if (mpi_rank == STARPU_MPI_PER_NODE)
 		{
 			mpi_rank = me;
@@ -158,20 +182,23 @@ void _starpu_mpi_exchange_data_after_execution(starpu_data_handle_t data, enum s
 		{
 			if (xrank != -1 && (xrank != STARPU_MPI_PER_NODE && me != xrank))
 			{
-				_STARPU_MPI_DEBUG(1, "Receive data %p back from the task %d which executed the codelet ...\n", data, xrank);
+				_STARPU_MPI_DEBUG(1, "Receive data %p back from the task %d which executed the codelet with prio %d...\n", data, xrank, prio);
 				if(data_tag == -1)
 					_STARPU_ERROR("StarPU needs to be told the MPI tag of this data, using starpu_mpi_data_register\n");
-				starpu_mpi_irecv_detached(data, xrank, data_tag, comm, NULL, NULL);
+				int ret = starpu_mpi_irecv_detached_prio(data, xrank, data_tag, prio, comm, NULL, NULL);
+				if (ret)
+					return ret;
 			}
 		}
 		else if (do_execute)
 		{
 			if(data_tag == -1)
 				_STARPU_ERROR("StarPU needs to be told the MPI tag of this data, using starpu_mpi_data_register\n");
-			_STARPU_MPI_DEBUG(1, "Send data %p back to its owner %d...\n", data, mpi_rank);
+			_STARPU_MPI_DEBUG(1, "Send data %p back to its owner %d with prio %d...\n", data, mpi_rank, prio);
 			_SEND_DATA(data, mode, mpi_rank, data_tag, prio, comm, NULL, NULL);
 		}
 	}
+	return 0;
 }
 
 static
@@ -179,7 +206,7 @@ void _starpu_mpi_clear_data_after_execution(starpu_data_handle_t data, enum star
 {
 	if (_starpu_cache_enabled)
 	{
-		if (mode & STARPU_W || mode & STARPU_REDUX)
+		if ((mode & STARPU_W && !(mode & STARPU_MPI_REDUX)) || mode & STARPU_REDUX)
 		{
 			/* The data has been modified, it MUST be removed from the cache */
 			starpu_mpi_cached_send_clear(data);
@@ -189,7 +216,7 @@ void _starpu_mpi_clear_data_after_execution(starpu_data_handle_t data, enum star
 	else
 	{
 		/* We allocated a temporary buffer for the received data, now drop it */
-		if ((mode & STARPU_R) && do_execute)
+		if ((mode & STARPU_R && !(mode & STARPU_MPI_REDUX)) && do_execute)
 		{
 			int mpi_rank = starpu_mpi_data_get_rank(data);
 			if (mpi_rank == STARPU_MPI_PER_NODE)
@@ -229,15 +256,20 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 	while ((arg_type = va_arg(varg_list_copy, int)) != 0)
 	{
 		int arg_type_nocommute = arg_type & ~STARPU_COMMUTE;
+
 		if (arg_type==STARPU_EXECUTE_ON_NODE)
 		{
-			*xrank = va_arg(varg_list_copy, int);
-			if (node_selected == 0)
+			int rank = va_arg(varg_list_copy, int);
+			if (rank != -1)
 			{
-				_STARPU_MPI_DEBUG(100, "Executing on node %d\n", *xrank);
-				*do_execute = 1;
-				node_selected = 1;
-				inconsistent_execute = 0;
+				*xrank = rank;
+				if (node_selected == 0)
+				{
+					_STARPU_MPI_DEBUG(100, "Executing on node %d\n", *xrank);
+					*do_execute = 1;
+					node_selected = 1;
+					inconsistent_execute = 0;
+				}
 			}
 		}
 		else if (arg_type==STARPU_EXECUTE_ON_DATA)
@@ -254,7 +286,7 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 				inconsistent_execute = 0;
 			}
 		}
-		else if (arg_type_nocommute & STARPU_R || arg_type_nocommute & STARPU_W || arg_type_nocommute & STARPU_RW || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX)
+		else if (arg_type_nocommute & STARPU_R || arg_type_nocommute & STARPU_W || arg_type_nocommute & STARPU_RW || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX || arg_type & STARPU_MPI_REDUX)
 		{
 			starpu_data_handle_t data = va_arg(varg_list_copy, starpu_data_handle_t);
 			enum starpu_data_access_mode mode = (enum starpu_data_access_mode) arg_type;
@@ -280,7 +312,7 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 		}
 		else if (arg_type == STARPU_DATA_ARRAY)
 		{
-			starpu_data_handle_t *datas = va_arg(varg_list_copy, starpu_data_handle_t *);
+			starpu_data_handle_t *data = va_arg(varg_list_copy, starpu_data_handle_t *);
 			int nb_handles = va_arg(varg_list_copy, int);
 			int i;
 
@@ -290,7 +322,7 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 				enum starpu_data_access_mode mode = STARPU_CODELET_GET_MODE(codelet, nb_data);
 				if (node_selected == 0)
 				{
-					int ret = _starpu_mpi_find_executee_node(datas[i], mode, me, do_execute, &inconsistent_execute, xrank);
+					int ret = _starpu_mpi_find_executee_node(data[i], mode, me, do_execute, &inconsistent_execute, xrank);
 					if (ret == -EINVAL)
 					{
 						free(descrs);
@@ -304,7 +336,7 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 					nb_allocated_data *= 2;
 					_STARPU_MPI_REALLOC(descrs, nb_allocated_data * sizeof(struct starpu_data_descr));
 				}
-				descrs[nb_data].handle = datas[i];
+				descrs[nb_data].handle = data[i];
 				descrs[nb_data].mode = mode;
 				nb_data ++;
 			}
@@ -386,6 +418,14 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 		{
 			(void)va_arg(varg_list_copy, void *);
 		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK)
+		{
+			(void)va_arg(varg_list_copy, _starpu_callback_func_t);
+		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK_ARG)
+		{
+			(void)va_arg(varg_list_copy, void *);
+		}
 		else if (arg_type==STARPU_PRIORITY)
 		{
 			prio = va_arg(varg_list_copy, int);
@@ -411,28 +451,28 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 			(void)va_arg(varg_list_copy, unsigned);
 		}
 		else if (arg_type==STARPU_PROLOGUE_CALLBACK)
-                {
+		{
 			(void)va_arg(varg_list_copy, _starpu_callback_func_t);
 		}
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG)
-                {
-                        (void)va_arg(varg_list_copy, void *);
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG_NFREE)
-                {
-                        (void)va_arg(varg_list_copy, void *);
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP)
-                {
-			(void)va_arg(varg_list_copy, _starpu_callback_func_t);
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG)
-                {
-                        (void)va_arg(varg_list_copy, void *);
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG)
+		{
+			(void)va_arg(varg_list_copy, void *);
 		}
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE)
-                {
-                        (void)va_arg(varg_list_copy, void *);
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG_NFREE)
+		{
+			(void)va_arg(varg_list_copy, void *);
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP)
+		{
+			(void)va_arg(varg_list_copy, _starpu_callback_func_t);
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG)
+		{
+			(void)va_arg(varg_list_copy, void *);
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE)
+		{
+			(void)va_arg(varg_list_copy, void *);
 		}
 		else if (arg_type==STARPU_EXECUTE_WHERE)
 		{
@@ -469,17 +509,40 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 			select_node_policy = va_arg(varg_list_copy, int);
 		}
 		else if (arg_type==STARPU_TASK_COLOR)
-                {
-                        (void)va_arg(varg_list_copy, int);
-                }
+		{
+			(void)va_arg(varg_list_copy, int);
+		}
 		else if (arg_type==STARPU_TASK_SYNCHRONOUS)
-                {
-                        (void)va_arg(varg_list_copy, int);
-                }
+		{
+			(void)va_arg(varg_list_copy, int);
+		}
+		else if (arg_type==STARPU_TRANSACTION)
+		{
+			(void)va_arg(varg_list_copy, struct starpu_transaction *);
+		}
 		else if (arg_type==STARPU_HANDLES_SEQUENTIAL_CONSISTENCY)
-                {
-                        (void)va_arg(varg_list_copy, char *);
-                }
+		{
+			(void)va_arg(varg_list_copy, char *);
+		}
+#ifdef STARPU_BUBBLE
+		else if (arg_type==STARPU_BUBBLE_FUNC)
+		{
+			STARPU_ASSERT_MSG(0, "Bubbles + MPI not supported yet\n");
+			(void)va_arg(varg_list,void*);
+		}
+		else if (arg_type==STARPU_BUBBLE_FUNC_ARG)
+		{
+			(void)va_arg(varg_list,void*);
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC)
+		{
+			(void)va_arg(varg_list,void*);
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC_ARG)
+		{
+			(void)va_arg(varg_list,void*);
+		}
+#endif
 		else if (arg_type==STARPU_TASK_END_DEP)
 		{
 			(void)va_arg(varg_list_copy, int);
@@ -505,6 +568,14 @@ int _starpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_nod
 		{
 			(void)va_arg(varg_list_copy, void *);
 		}
+		else if (arg_type==STARPU_TASK_FILE)
+		{
+			(void)va_arg(varg_list_copy, const char *);
+		}
+		else if (arg_type==STARPU_TASK_LINE)
+		{
+			(void)va_arg(varg_list_copy, int);
+		}
 		else
 		{
 			STARPU_ABORT_MSG("Unrecognized argument %d, did you perhaps forget to end arguments with 0?\n", arg_type);
@@ -541,7 +612,7 @@ int _starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, stru
 	int me, do_execute, xrank, nb_nodes;
 	int ret;
 	int i;
-	struct starpu_data_descr *descrs;
+	struct starpu_data_descr *descrs = NULL;
 	int nb_data;
 	int prio;
 
@@ -559,6 +630,14 @@ int _starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, stru
 	/* Send and receive data as requested */
 	for(i=0 ; i<nb_data ; i++)
 	{
+                if (descrs[i].handle && descrs[i].handle->mpi_data)
+		{
+			char *redux_map = starpu_mpi_data_get_redux_map(descrs[i].handle);
+			if (redux_map != NULL && descrs[i].mode & STARPU_R && descrs[i].mode & ~ STARPU_REDUX && descrs[i].mode & ~ STARPU_MPI_REDUX)
+			{
+				_starpu_mpi_redux_wrapup_data(descrs[i].handle);
+			}
+		}
 		_starpu_mpi_exchange_data_before_execution(descrs[i].handle, descrs[i].mode, me, xrank, do_execute, prio, comm);
 	}
 
@@ -574,14 +653,7 @@ int _starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, stru
 	else
 		free(descrs);
 
-
-	_STARPU_TRACE_TASK_MPI_PRE_END();
-
-	if (do_execute == 0)
-	{
-		return 1;
-	}
-	else
+	if (do_execute == 1)
 	{
 		va_list varg_list_copy;
 		_STARPU_MPI_DEBUG(100, "Execution of the codelet %p (%s)\n", codelet, codelet?codelet->name:NULL);
@@ -596,8 +668,31 @@ int _starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, stru
 		_starpu_task_insert_create(codelet, *task, varg_list_copy);
 		va_end(varg_list_copy);
 
-		return 0;
+		if ((*task)->cl)
+		{
+			/* we suppose the current context is not going to change between now and the execution of the task */
+			(*task)->sched_ctx = _starpu_sched_ctx_get_current_context();
+			/* Check the type of worker(s) required by the task exist */
+			if (STARPU_UNLIKELY(!_starpu_worker_exists(*task)))
+			{
+				_STARPU_MPI_DEBUG(0, "There is no worker to execute the codelet %p (%s)\n", codelet, codelet?codelet->name:NULL);
+				return -ENODEV;
+			}
+
+			/* In case we require that a task should be explicitly
+			 * executed on a specific worker, we make sure that the worker
+			 * is able to execute this task.  */
+			if (STARPU_UNLIKELY((*task)->execute_on_a_specific_worker && !starpu_combined_worker_can_execute_task((*task)->workerid, *task, 0)))
+			{
+				_STARPU_MPI_DEBUG(0, "The specified worker %d cannot execute the codelet %p (%s)\n", (*task)->workerid, codelet, codelet?codelet->name:NULL);
+				return -ENODEV;
+			}
+		}
 	}
+
+	_STARPU_TRACE_TASK_MPI_PRE_END();
+
+	return do_execute;
 }
 
 int _starpu_mpi_task_postbuild_v(MPI_Comm comm, int xrank, int do_execute, struct starpu_data_descr *descrs, int nb_data, int prio)
@@ -609,12 +704,45 @@ int _starpu_mpi_task_postbuild_v(MPI_Comm comm, int xrank, int do_execute, struc
 
 	for(i=0 ; i<nb_data ; i++)
 	{
+		if ((descrs[i].mode & STARPU_REDUX || descrs[i].mode & STARPU_MPI_REDUX) && descrs[i].handle)
+		{
+			struct _starpu_mpi_data *mpi_data = (struct _starpu_mpi_data *) descrs[i].handle->mpi_data;
+			int rrank = starpu_mpi_data_get_rank(descrs[i].handle);
+			int size;
+			starpu_mpi_comm_size(comm, &size);
+			if (mpi_data->redux_map == NULL)
+			{
+				_STARPU_CALLOC(mpi_data->redux_map, size, sizeof(mpi_data->redux_map[0]));
+			}
+			mpi_data->redux_map [xrank] = 1;
+			mpi_data->redux_map [rrank] = 1;
+			int outside_owner = 0;
+			int j;
+			for (j = 0; j < size; j++)
+			{
+				if (mpi_data->redux_map[j] && j != rrank)
+				{
+					outside_owner = 1;
+					break;
+				}
+			}
+			if (outside_owner)
+			{
+				struct _starpu_redux_data_entry *entry;
+				HASH_FIND_PTR(_redux_data, &descrs[i].handle, entry);
+				if (entry == NULL)
+				{
+					_STARPU_MPI_MALLOC(entry, sizeof(*entry));
+					starpu_data_handle_t data_handle = descrs[i].handle;
+					entry->data_handle = data_handle;
+					HASH_ADD_PTR(_redux_data, data_handle, entry);
+				}
+			}
+		}
 		_starpu_mpi_exchange_data_after_execution(descrs[i].handle, descrs[i].mode, me, xrank, do_execute, prio, comm);
 		_starpu_mpi_clear_data_after_execution(descrs[i].handle, descrs[i].mode, me, do_execute);
 	}
 
-	free(descrs);
-
 	_STARPU_TRACE_TASK_MPI_POST_END();
 	_STARPU_MPI_LOG_OUT();
 	return 0;
@@ -635,14 +763,14 @@ int _starpu_mpi_task_insert_v(MPI_Comm comm, struct starpu_codelet *codelet, va_
 	if (ret < 0)
 		return ret;
 
-	if (ret == 0)
+	if (ret == 1)
 	{
 		do_execute = 1;
 		ret = starpu_task_submit(task);
 
 		if (STARPU_UNLIKELY(ret == -ENODEV))
 		{
-			_STARPU_MSG("submission of task %p wih codelet %p failed (symbol `%s') (err: ENODEV)\n",
+			_STARPU_MSG("submission of task %p with codelet %p failed (symbol `%s') (err: ENODEV)\n",
 				    task, task->cl,
 				    (codelet == NULL) ? "none" :
 				    task->cl->name ? task->cl->name :
@@ -650,17 +778,21 @@ int _starpu_mpi_task_insert_v(MPI_Comm comm, struct starpu_codelet *codelet, va_
 
 			task->destroy = 0;
 			starpu_task_destroy(task);
+			free(descrs);
+			return -ENODEV;
 		}
 	}
 
 	int val = _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+	free(descrs);
 
-	if (ret == 0 && pre_submit_hook)
-		pre_submit_hook(task);
+	if (ret == 1)
+		_starpu_mpi_pre_submit_hook_call(task);
 
 	return val;
 }
 
+#undef starpu_mpi_task_insert
 int starpu_mpi_task_insert(MPI_Comm comm, struct starpu_codelet *codelet, ...)
 {
 	va_list varg_list;
@@ -672,6 +804,7 @@ int starpu_mpi_task_insert(MPI_Comm comm, struct starpu_codelet *codelet, ...)
 	return ret;
 }
 
+#undef starpu_mpi_insert_task
 int starpu_mpi_insert_task(MPI_Comm comm, struct starpu_codelet *codelet, ...)
 {
 	va_list varg_list;
@@ -683,6 +816,7 @@ int starpu_mpi_insert_task(MPI_Comm comm, struct starpu_codelet *codelet, ...)
 	return ret;
 }
 
+#undef starpu_mpi_task_build
 struct starpu_task *starpu_mpi_task_build(MPI_Comm comm, struct starpu_codelet *codelet, ...)
 {
 	va_list varg_list;
@@ -692,8 +826,16 @@ struct starpu_task *starpu_mpi_task_build(MPI_Comm comm, struct starpu_codelet *
 	va_start(varg_list, codelet);
 	ret = _starpu_mpi_task_build_v(comm, codelet, &task, NULL, NULL, NULL, NULL, varg_list);
 	va_end(varg_list);
-	STARPU_ASSERT(ret >= 0);
-	return (ret > 0) ? NULL : task;
+	return (ret == 1 || ret == -ENODEV) ? task : NULL;
+}
+
+struct starpu_task *starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, va_list varg_list)
+{
+	struct starpu_task *task;
+	int ret;
+
+	ret = _starpu_mpi_task_build_v(comm, codelet, &task, NULL, NULL, NULL, NULL, varg_list);
+	return (ret == 1 || ret == -ENODEV) ? task : NULL;
 }
 
 int starpu_mpi_task_post_build(MPI_Comm comm, struct starpu_codelet *codelet, ...)
@@ -715,9 +857,98 @@ int starpu_mpi_task_post_build(MPI_Comm comm, struct starpu_codelet *codelet, ..
 	if (ret < 0)
 		return ret;
 
-	return _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+	ret = _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+	free(descrs);
+	return ret;
 }
 
+int starpu_mpi_task_post_build_v(MPI_Comm comm, struct starpu_codelet *codelet, va_list varg_list)
+{
+	int xrank, do_execute;
+	int ret, me, nb_nodes;
+	struct starpu_data_descr *descrs;
+	int nb_data;
+	int prio;
+
+	starpu_mpi_comm_rank(comm, &me);
+	starpu_mpi_comm_size(comm, &nb_nodes);
+
+	/* Find out whether we are to execute the data because we own the data to be written to. */
+	ret = _starpu_mpi_task_decode_v(codelet, me, nb_nodes, &xrank, &do_execute, &descrs, &nb_data, &prio, varg_list);
+	if (ret < 0)
+		return ret;
+
+	ret = _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+	free(descrs);
+	return ret;
+}
+
+int starpu_mpi_task_exchange_data_before_execution(MPI_Comm comm, struct starpu_task *task, struct starpu_data_descr *descrs, struct starpu_mpi_task_exchange_params *params)
+{
+	int me, nb_nodes, inconsistent_execute;
+	unsigned i;
+	int select_node_policy = STARPU_MPI_NODE_SELECTION_CURRENT_POLICY;
+	unsigned nb_data;
+
+	nb_data = STARPU_TASK_GET_NBUFFERS(task);
+	starpu_mpi_comm_rank(comm, &me);
+	starpu_mpi_comm_size(comm, &nb_nodes);
+	params->xrank = -1;
+	inconsistent_execute = 0;
+	for(i=0 ; i<nb_data ; i++)
+	{
+		descrs[i].handle = STARPU_TASK_GET_HANDLE(task, i);
+		descrs[i].mode = STARPU_TASK_GET_MODE(task, i);
+
+		int ret = _starpu_mpi_find_executee_node(descrs[i].handle,
+							 descrs[i].mode,
+							 me, &(params->do_execute),
+							 &inconsistent_execute, &(params->xrank));
+		if (ret == -EINVAL)
+		{
+			return ret;
+		}
+	}
+	if (inconsistent_execute == 1 || params->xrank == -1)
+	{
+		// We need to find out which node is going to execute the codelet.
+		_STARPU_MPI_DEBUG(100, "Different nodes are owning W data. The node to execute the codelet is going to be selected with the current selection node policy. See starpu_mpi_node_selection_set_current_policy() to change the policy, or use STARPU_EXECUTE_ON_NODE or STARPU_EXECUTE_ON_DATA to specify the node\n");
+		params->xrank = _starpu_mpi_select_node(me, nb_nodes, descrs, nb_data, select_node_policy);
+		params->do_execute = (params->xrank == STARPU_MPI_PER_NODE) || (me == params->xrank);
+	}
+	else
+	{
+		_STARPU_MPI_DEBUG(100, "Inconsistent=%d - xrank=%d\n", inconsistent_execute, params->xrank);
+		params->do_execute = (params->xrank == STARPU_MPI_PER_NODE) || (me == params->xrank);
+	}
+
+	for(i=0 ; i<nb_data ; i++)
+	{
+		_starpu_mpi_exchange_data_before_execution(descrs[i].handle,
+							   descrs[i].mode,
+							   me,
+							   params->xrank,
+							   params->do_execute,
+							   task->priority,
+							   comm);
+	}
+
+	params->priority = task->priority;
+	return 0;
+}
+
+int starpu_mpi_task_exchange_data_after_execution(MPI_Comm comm, struct starpu_data_descr *descrs, unsigned nb_data, struct starpu_mpi_task_exchange_params params)
+{
+	return _starpu_mpi_task_postbuild_v(comm, params.xrank, params.do_execute, descrs, nb_data, params.priority);
+}
+
+struct starpu_codelet _starpu_mpi_redux_data_synchro_cl =
+{
+	.where = STARPU_NOWHERE,
+	.modes = {STARPU_R, STARPU_W},
+	.nbuffers = 2
+};
+
 struct _starpu_mpi_redux_data_args
 {
 	starpu_data_handle_t data_handle;
@@ -726,170 +957,260 @@ struct _starpu_mpi_redux_data_args
 	int node;
 	MPI_Comm comm;
 	struct starpu_task *taskB;
+	long taskC_jobid;
 };
 
-void _starpu_mpi_redux_data_dummy_func(void *buffers[], void *cl_arg)
+void _starpu_mpi_redux_fill_post_sync_jobid(const void * const redux_data_args, long * const post_sync_jobid)
 {
-	(void)buffers;
-	(void)cl_arg;
+	*post_sync_jobid = ((const struct _starpu_mpi_redux_data_args *) redux_data_args)->taskC_jobid;
 }
 
-/* Dummy cost function for simgrid */
-static double cost_function(struct starpu_task *task, unsigned nimpl)
-{
-	(void)task;
-	(void)nimpl;
-	return 0.000001;
-}
-static struct starpu_perfmodel dumb_model =
-{
-	.type		= STARPU_COMMON,
-	.cost_function	= cost_function
-};
-
-static
-struct starpu_codelet _starpu_mpi_redux_data_read_cl =
-{
-	.cpu_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.cuda_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.opencl_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.nbuffers = 1,
-	.modes = {STARPU_R},
-	.model = &dumb_model,
-	.name = "_starpu_mpi_redux_data_read_cl"
-};
-
-struct starpu_codelet _starpu_mpi_redux_data_readwrite_cl =
-{
-	.cpu_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.cuda_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.opencl_funcs = {_starpu_mpi_redux_data_dummy_func},
-	.nbuffers = 1,
-	.modes = {STARPU_RW},
-	.model = &dumb_model,
-	.name = "_starpu_mpi_redux_data_write_cl"
-};
-
-static
-void _starpu_mpi_redux_data_detached_callback(void *arg)
-{
-	struct _starpu_mpi_redux_data_args *args = (struct _starpu_mpi_redux_data_args *) arg;
-
-	STARPU_TASK_SET_HANDLE(args->taskB, args->new_handle, 1);
-	int ret = starpu_task_submit(args->taskB);
-	STARPU_ASSERT(ret == 0);
-
-	starpu_data_unregister_submit(args->new_handle);
-	free(args);
-}
-
-static
-void _starpu_mpi_redux_data_recv_callback(void *callback_arg)
-{
-	struct _starpu_mpi_redux_data_args *args = (struct _starpu_mpi_redux_data_args *) callback_arg;
-	starpu_data_register_same(&args->new_handle, args->data_handle);
-
-	starpu_mpi_irecv_detached_sequential_consistency(args->new_handle, args->node, args->data_tag, args->comm, _starpu_mpi_redux_data_detached_callback, args, 0);
-}
-
-/* TODO: this should rather be implicitly called by starpu_mpi_task_insert when
- * a data previously accessed in REDUX mode gets accessed in R mode. */
-void starpu_mpi_redux_data_prio(MPI_Comm comm, starpu_data_handle_t data_handle, int prio)
+int starpu_mpi_redux_data_prio_tree(MPI_Comm comm, starpu_data_handle_t data_handle, int prio, int arity)
 {
 	int me, rank, nb_nodes;
-	starpu_mpi_tag_t tag;
+	starpu_mpi_tag_t data_tag;
 
 	rank = starpu_mpi_data_get_rank(data_handle);
-	tag = starpu_mpi_data_get_tag(data_handle);
+	data_tag = starpu_mpi_data_get_tag(data_handle);
+	struct _starpu_mpi_data *mpi_data = data_handle->mpi_data;
 	if (rank == -1)
 	{
 		_STARPU_ERROR("StarPU needs to be told the MPI rank of this data, using starpu_mpi_data_register\n");
 	}
-	if (tag == -1)
+	if (data_tag == -1)
 	{
 		_STARPU_ERROR("StarPU needs to be told the MPI tag of this data, using starpu_mpi_data_register\n");
 	}
-
+	if (mpi_data->redux_map == NULL)
+	{
+		_STARPU_MPI_DEBUG(5, "I do not contribute to this reduction\n");
+		return 0;
+	}
 	starpu_mpi_comm_rank(comm, &me);
 	starpu_mpi_comm_size(comm, &nb_nodes);
+	struct _starpu_redux_data_entry *entry;
+	HASH_FIND_PTR(_redux_data, &data_handle, entry);
 
-	_STARPU_MPI_DEBUG(1, "Doing reduction for data %p on node %d with %d nodes ...\n", data_handle, rank, nb_nodes);
-
-	// need to count how many nodes have the data in redux mode
-	if (me == rank)
+#ifdef STARPU_MPI_VERBOSE
+	int current_level=0;
+#endif
+	int nb_contrib, next_nb_contrib;
+	int i, j, step, node;
+	char root_in_step, me_in_step;
+	// https://stackoverflow.com/questions/109023/how-to-count-the-number-of-set-bits-in-a-32-bit-integer
+	// https://stackoverflow.com/a/109025
+	// see hamming weight
+	//nb_contrib = std::popcount(mpi_data->redux_map); // most preferable
+	nb_contrib=0;
+	for (i=0;i<nb_nodes;i++)
 	{
-		int i;
-
-		// taskC depends on all taskBs created
-		struct starpu_task *taskC = starpu_task_create();
-		taskC->cl = &_starpu_mpi_redux_data_readwrite_cl;
-		STARPU_TASK_SET_HANDLE(taskC, data_handle, 0);
-
-		for(i=0 ; i<nb_nodes ; i++)
+		_STARPU_MPI_DEBUG(5, "mpi_data->redux_map[%d] = %d\n", i, mpi_data->redux_map[i]);
+		if (mpi_data->redux_map[i]) nb_contrib++;
+	}
+	if (nb_contrib < 2)
+	{
+		_STARPU_MPI_DEBUG(5, "Not enough contributors to create a n-ary reduction tree.\n");
+		/* duplicated at the end of this function */
+		if (entry != NULL)
 		{
-			if (i != rank)
-			{
-				/* We need to make sure all is
-				 * executed after data_handle finished
-				 * its last read access, we hence do
-				 * the following:
-				 * - submit an empty task A reading
-				 * data_handle whose callback submits
-				 * the mpi comm with sequential
-				 * consistency set to 0, whose
-				 * callback submits the redux_cl task
-				 * B with sequential consistency set
-				 * to 0,
-				 * - submit an empty task C reading
-				 * and writing data_handle and
-				 * depending on task B, just to replug
-				 * with implicit data dependencies
-				 * with tasks inserted after this
-				 * reduction.
-				 */
-
-				struct _starpu_mpi_redux_data_args *args;
-				_STARPU_MPI_MALLOC(args, sizeof(struct _starpu_mpi_redux_data_args));
-				args->data_handle = data_handle;
-				args->data_tag = tag;
-				args->node = i;
-				args->comm = comm;
-
-				// We need to create taskB early as
-				// taskC declares a dependancy on it
-				args->taskB = starpu_task_create();
-				args->taskB->cl = args->data_handle->redux_cl;
-				args->taskB->sequential_consistency = 0;
-				STARPU_TASK_SET_HANDLE(args->taskB, args->data_handle, 0);
-
-				starpu_task_declare_deps_array(taskC, 1, &args->taskB);
-
-				// Submit taskA
-				starpu_task_insert(&_starpu_mpi_redux_data_read_cl,
-						   STARPU_R, data_handle,
-						   STARPU_CALLBACK_WITH_ARG_NFREE, _starpu_mpi_redux_data_recv_callback, args,
-						   0);
-			}
+			HASH_DEL(_redux_data, entry);
+			free(entry);
 		}
-
-		int ret = starpu_task_submit(taskC);
-		STARPU_ASSERT(ret == 0);
+		free(mpi_data->redux_map);
+		mpi_data->redux_map = NULL;
+		return 0;
 	}
-	else
+	if (arity < 2)
 	{
-		_STARPU_MPI_DEBUG(1, "Sending redux handle to %d ...\n", rank);
-		starpu_mpi_isend_detached_prio(data_handle, rank, tag, prio, comm, NULL, NULL);
-		starpu_task_insert(data_handle->init_cl, STARPU_W, data_handle, 0);
+		arity = nb_contrib;
+	}
+	arity = STARPU_MIN(arity,nb_contrib);
+	_STARPU_MPI_DEBUG(5, "There is %d contributors\n", nb_contrib);
+	int contributors[nb_contrib];
+	int reducing_node;
+	j=0;
+	for (i=0;i<nb_nodes;i++)
+	{
+		_STARPU_MPI_DEBUG(5, "%d in reduction ? %d\n", i, mpi_data->redux_map[i]);
+		if (mpi_data->redux_map[i])
+		{
+			contributors[j++] = i;
+		}
+	}
+	for (i=0;i<nb_contrib;i++)
+	{
+		_STARPU_MPI_DEBUG(5, "%dth contributor = %d\n", i, contributors[i]);
 	}
-	/* FIXME: In order to prevent simultaneous receive submissions
-	 * on the same handle, we need to wait that all the starpu_mpi
-	 * tasks are done before submitting next tasks. The current
-	 * version of the implementation does not support multiple
-	 * simultaneous receive requests on the same handle.*/
-	starpu_task_wait_for_all();
 
+	_STARPU_MPI_DEBUG(15, "mpi_redux _ STARTING with %d-ary tree \n", arity);
+	while (nb_contrib != 1)
+	{
+		_STARPU_MPI_DEBUG(5, "%dth level in the reduction \n", current_level);
+		if (nb_contrib%arity == 0) next_nb_contrib = nb_contrib/arity;
+		else next_nb_contrib = nb_contrib/arity + 1;
+		for (step = 0; step < next_nb_contrib; step++)
+		{
+			root_in_step = 0;
+			me_in_step = 0;
+			for (node = step*arity ; node < nb_contrib && node < (step+1)*arity ; node++)
+			{
+				if (contributors[node] == rank) root_in_step = 1;
+				if (contributors[node] == me) me_in_step = 1;
+			}
+			/* FIXME: if the root node is note in the step, then we agree the node
+			 * with the lowest id reduces the step : we could agree on another
+			 * node to better load balance in the case of multiple reductions involving
+			 * the same sets of nodes
+			 * FIX: We chose to use the tag%arity-th contributor in the step
+			 */
+			if (root_in_step)
+			{
+				reducing_node = rank;
+			}
+			else if (step*arity + data_tag%arity < nb_contrib)
+			{
+				reducing_node = contributors[step*arity + data_tag%arity];
+			}
+			else
+			{
+				reducing_node = contributors[step*arity];
+			}
+
+			if (me == reducing_node)
+			{
+				_STARPU_MPI_DEBUG(5, "mpi_redux _ %dth level, %dth step ; chose %d node\n", current_level, step, reducing_node);
+				for (node = step*arity ; node < nb_contrib && node < (step+1)*arity ; node++)
+				{
+					if (me != contributors[node])
+					{
+						_STARPU_MPI_DEBUG(5, "%d takes part in the reduction of %p towards %d (%dth level ; %dth step) \n",
+								  contributors[node], data_handle, reducing_node, current_level, step);
+						/* We need to make sure all is
+						 * executed after data_handle finished
+						 * its last read access, we hence do
+						 * the following:
+						 * - submit an empty task A reading
+						 * data_handle
+						 * - submit the reducing task B
+						 * reading and writing data_handle and
+						 * depending on task A through sequential
+						 * consistency
+						 */
+						starpu_data_handle_t new_handle;
+						starpu_data_register_same(&new_handle, data_handle);
+						/* Task A */
+				       	        int ret = starpu_task_insert(&_starpu_mpi_redux_data_synchro_cl,
+									     STARPU_R, data_handle,
+									     STARPU_W, new_handle,
+									     STARPU_PRIORITY, prio,
+									     STARPU_NAME, "redux_prio_tree_synchro_cl",
+									     0);
+						if (ret)
+							return ret;
+				       	        ret = starpu_mpi_irecv_detached_prio(new_handle, contributors[node], data_tag, prio, comm, NULL, NULL);
+						if (ret)
+							return ret;
+
+					        /* Task B */
+						ret = starpu_task_insert(data_handle->redux_cl,
+						                         STARPU_RW|STARPU_COMMUTE, data_handle,
+									 STARPU_R, new_handle,
+									 STARPU_PRIORITY, prio,
+									 STARPU_NAME, "redux_prio_tree_redux_cl",
+									 0);
+						if (ret)
+							return ret;
+						starpu_data_unregister_submit(new_handle);
+					}
+				}
+			}
+			else if (me_in_step)
+			{
+				_STARPU_MPI_DEBUG(5, "Sending redux handle to %d ...\n", reducing_node);
+				int ret = starpu_mpi_isend_detached_prio(data_handle, reducing_node, data_tag, prio, comm, NULL, NULL);
+				if (ret)
+					return ret;
+				starpu_data_invalidate_submit(data_handle);
+			}
+			contributors[step] = reducing_node;
+		}
+		nb_contrib = next_nb_contrib;
+#ifdef STARPU_MPI_VERBOSE
+		current_level++;
+#endif
+	}
+
+	/* duplicated when not enough contributors */
+	if (entry != NULL)
+	{
+		HASH_DEL(_redux_data, entry);
+		free(entry);
+	}
+	free(mpi_data->redux_map);
+	mpi_data->redux_map = NULL;
+	return 0;
 }
-void starpu_mpi_redux_data(MPI_Comm comm, starpu_data_handle_t data_handle)
+
+int starpu_mpi_redux_data(MPI_Comm comm, starpu_data_handle_t data_handle)
 {
 	return starpu_mpi_redux_data_prio(comm, data_handle, 0);
 }
+
+int starpu_mpi_redux_data_tree(MPI_Comm comm, starpu_data_handle_t data_handle, int arity)
+{
+	return starpu_mpi_redux_data_prio_tree(comm, data_handle, 0, arity);
+}
+
+int starpu_mpi_redux_data_prio(MPI_Comm comm, starpu_data_handle_t data_handle, int prio)
+{
+	int nb_nodes, nb_contrib, i;
+	struct _starpu_mpi_data *mpi_data = data_handle->mpi_data;
+	if (mpi_data->redux_map == NULL)
+	{
+		_STARPU_MPI_DEBUG(5, "I do not contribute to this reduction\n");
+		return 0;
+	}
+	starpu_mpi_comm_size(comm, &nb_nodes);
+	nb_contrib=0;
+	for (i=0;i<nb_nodes;i++)
+	{
+		if (mpi_data->redux_map[i])
+		{
+			nb_contrib++;
+		}
+	}
+	return starpu_mpi_redux_data_prio_tree(comm, data_handle, prio, nb_contrib);
+}
+
+void _starpu_mpi_redux_wrapup_data(starpu_data_handle_t data_handle)
+{
+	// We could check if the handle makes sense but we do not because it helps the programmer using coherent
+	// distributed-memory reduction patterns
+	size_t data_size = starpu_data_get_size(data_handle);
+ 	// Small data => flat tree | binary tree
+	int _starpu_mpi_redux_threshold = starpu_getenv_number_default("STARPU_MPI_REDUX_ARITY_THRESHOLD", 1024);
+	int _starpu_mpi_redux_tree_size = 2;
+	if (_starpu_mpi_redux_threshold < 0 || (_starpu_mpi_redux_threshold > 0 && data_size < (size_t) _starpu_mpi_redux_threshold))
+	{
+		_starpu_mpi_redux_tree_size = STARPU_MAXNODES;
+	}
+	struct _starpu_mpi_data *mpi_data = data_handle->mpi_data;
+	struct _starpu_redux_data_entry *entry;
+
+	HASH_FIND_PTR(_redux_data, &data_handle, entry);
+	if (entry != NULL)
+	{
+		starpu_mpi_redux_data_tree(mpi_data->node_tag.node.comm,data_handle,_starpu_mpi_redux_tree_size);
+	}
+	return;
+}
+
+void _starpu_mpi_redux_wrapup_data_all()
+{
+ 	struct _starpu_redux_data_entry *entry = NULL, *tmp = NULL;
+ 	HASH_ITER(hh, _redux_data, entry, tmp)
+	{
+		_starpu_mpi_redux_wrapup_data(entry->data_handle);
+	}
+ 	return;
+}
