@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -66,9 +66,20 @@ int main(int argc, char **argv)
 	int *vector = NULL;
 	starpu_data_handle_t *data_handles;
 	int size=10;
+	struct starpu_conf conf;
+	int mpi_init;
 
-	ret = starpu_mpi_init_conf(&argc, &argv, 1, MPI_COMM_WORLD, NULL);
+	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
+
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
+
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &nodes);
 
@@ -77,13 +88,15 @@ int main(int argc, char **argv)
 		if (rank == 0)
 			FPRINTF(stderr, "We need at least 1 CPU worker.\n");
 		starpu_mpi_shutdown();
+		if (!mpi_init)
+			MPI_Finalize();
 		return rank == 0 ? STARPU_TEST_SKIPPED : 0;
 	}
 
 	if (rank == 0)
 	{
 		/* Allocate the vector */
-		vector = malloc(size * sizeof(int));
+		starpu_malloc((void **)&vector, size * sizeof(int));
 		for(x=0 ; x<size ; x++)
 			vector[x] = x+10;
 
@@ -130,7 +143,8 @@ int main(int argc, char **argv)
 			starpu_mpi_data_set_rank(data_handles[x], mpi_rank);
 		}
 	}
-	starpu_mpi_scatter_detached(data_handles, size, 0, MPI_COMM_WORLD, scallback, "scatter", NULL, NULL);
+	ret = starpu_mpi_scatter_detached(data_handles, size, 0, MPI_COMM_WORLD, scallback, "scatter", NULL, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_scatter_detached");
 
 	/* Calculation */
 	for(x = 0; x < size ; x++)
@@ -141,16 +155,18 @@ int main(int argc, char **argv)
 			if (owner == rank)
 			{
 				FPRINTF_MPI(stderr,"Computing on data[%d]\n", x);
-				starpu_task_insert(&cl,
+				ret = starpu_task_insert(&cl,
 						   STARPU_VALUE, &rank, sizeof(rank),
 						   STARPU_RW, data_handles[x],
 						   0);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 			}
 		}
 	}
 
 	/* Gather the matrix on main node */
-	starpu_mpi_gather_detached(data_handles, size, 0, MPI_COMM_WORLD, scallback, "gather", rcallback, "gather");
+	ret = starpu_mpi_gather_detached(data_handles, size, 0, MPI_COMM_WORLD, scallback, "gather", rcallback, "gather");
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_gather_detached");
 	for(x = 0; x < size ; x++)
 	{
 		if (data_handles[x])
@@ -186,12 +202,15 @@ int main(int argc, char **argv)
 				ret = 1;
 			}
 		}
-		free(vector);
+		starpu_free_noflag(vector, size * sizeof(int));
 	}
 
 	// Free memory
 	free(data_handles);
 
 	starpu_mpi_shutdown();
+	if (!mpi_init)
+		MPI_Finalize();
+
 	return (rank == 0) ? ret : 0;
 }
