@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,8 +24,7 @@
 #include <math.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -39,7 +38,7 @@ struct sc_hypervisor_policy
 	   policy, the policy corresponding to this name will be used
 	   by the hypervisor
 	*/
-	const char* name;
+	const char *name;
 
 	/**
 	   Indicate whether the policy is custom or not
@@ -47,15 +46,15 @@ struct sc_hypervisor_policy
 	unsigned custom;
 
 	/**
-	   Distribute workers to contexts even at the begining of the
+	   Distribute workers to contexts even at the beginning of the
 	   program
 	*/
-	void (*size_ctxs)(unsigned *sched_ctxs, int nsched_ctxs , int *workers, int nworkers);
+	void (*size_ctxs)(unsigned *sched_ctxs, int nsched_ctxs, int *workers, int nworkers);
 
 	/**
 	   Require explicit resizing
 	*/
-	void (*resize_ctxs)(unsigned *sched_ctxs, int nsched_ctxs , int *workers, int nworkers);
+	void (*resize_ctxs)(unsigned *sched_ctxs, int nsched_ctxs, int *workers, int nworkers);
 
 	/**
 	   Called whenever the indicated worker executes another idle
@@ -73,7 +72,7 @@ struct sc_hypervisor_policy
 	   Called whenever a task is poped from the worker’s queue
 	   corresponding to the context sched_ctx
 	*/
-	void (*handle_poped_task)(unsigned sched_ctx, int worker,struct starpu_task *task, uint32_t footprint);
+	void (*handle_poped_task)(unsigned sched_ctx, int worker, struct starpu_task *task, uint32_t footprint);
 
 	/**
 	   Called whenever a task is executed on the indicated worker
@@ -101,7 +100,7 @@ struct sc_hypervisor_policy
 
 	/**
 	   the hypervisor takes a decision when a certain ctx was
-	   registerd
+	   registered
 	*/
 	void (*start_ctx)(unsigned sched_ctx);
 
@@ -147,7 +146,7 @@ extern starpu_pthread_mutex_t act_hypervisor_mutex;
 /**
    Start the hypervisor with the given policy
 */
-void* sc_hypervisor_init(struct sc_hypervisor_policy *policy);
+void *sc_hypervisor_init(struct sc_hypervisor_policy *policy);
 
 /**
    Shutdown the hypervisor.
@@ -177,11 +176,11 @@ void sc_hypervisor_unregister_ctx(unsigned sched_ctx);
 void sc_hypervisor_post_resize_request(unsigned sched_ctx, int task_tag);
 
 /**
-   Require reconsidering the distribution of ressources over the
+   Require reconsidering the distribution of resources over the
    indicated scheduling contexts, i.e reevaluate the distribution of
    the resources and eventually resize if needed
 */
-void sc_hypervisor_resize_ctxs(unsigned *sched_ctxs, int nsched_ctxs , int *workers, int nworkers);
+void sc_hypervisor_resize_ctxs(unsigned *sched_ctxs, int nsched_ctxs, int *workers, int nworkers);
 
 /**
    Do not allow the hypervisor to resize a context.
@@ -196,17 +195,17 @@ void sc_hypervisor_start_resize(unsigned sched_ctx);
 /**
    Return the name of the resizing policy used by the hypervisor
 */
-const char *sc_hypervisor_get_policy();
+const char *sc_hypervisor_get_policy(void);
 
 /**
    Ask the hypervisor to add workers to a sched_ctx
 */
-void sc_hypervisor_add_workers_to_sched_ctx(int* workers_to_add, unsigned nworkers_to_add, unsigned sched_ctx);
+void sc_hypervisor_add_workers_to_sched_ctx(int *workers_to_add, unsigned nworkers_to_add, unsigned sched_ctx);
 
 /**
    Ask the hypervisor to remove workers from a sched_ctx
 */
-void sc_hypervisor_remove_workers_from_sched_ctx(int* workers_to_remove, unsigned nworkers_to_remove, unsigned sched_ctx, unsigned now);
+void sc_hypervisor_remove_workers_from_sched_ctx(int *workers_to_remove, unsigned nworkers_to_remove, unsigned sched_ctx, unsigned now);
 
 /**
    Ask the hypervisor to move workers from one context to another
@@ -222,7 +221,7 @@ void sc_hypervisor_size_ctxs(unsigned *sched_ctxs, int nsched_ctxs, int *workers
 /**
    Check if there are pending demands of resizing
 */
-unsigned sc_hypervisor_get_size_req(unsigned **sched_ctxs, int* nsched_ctxs, int **workers, int *nworkers);
+unsigned sc_hypervisor_get_size_req(unsigned **sched_ctxs, int *nsched_ctxs, int **workers, int *nworkers);
 
 /**
    Save a demand of resizing
@@ -279,11 +278,11 @@ unsigned sc_hypervisor_get_nhierarchy_levels(void);
 void sc_hypervisor_get_leaves(unsigned *sched_ctxs, int nsched_ctxs, unsigned *leaves, int *nleaves);
 
 /**
-   Return the nready flops of all ctxs below in hierachy of sched_ctx
+   Return the nready flops of all ctxs below in hierarchy of sched_ctx
 */
 double sc_hypervisor_get_nready_flops_of_all_sons_of_sched_ctx(unsigned sched_ctx);
 
-void sc_hypervisor_print_overhead();
+void sc_hypervisor_print_overhead(void);
 
 void sc_hypervisor_init_worker(int workerid, unsigned sched_ctx);
 
