@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,8 +15,14 @@
  */
 
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+static int map_matrix(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int unmap_matrix(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int update_map_matrix(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
 
 static const struct starpu_data_copy_methods matrix_copy_data_methods_s =
 {
@@ -24,11 +30,12 @@ static const struct starpu_data_copy_methods matrix_copy_data_methods_s =
 };
 
 static void matrix_init(void *data_interface);
-static void register_matrix_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_matrix_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static void *matrix_to_pointer(void *data_interface, unsigned node);
-static int matrix_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static starpu_ssize_t allocate_matrix_buffer_on_node(void *data_interface_, unsigned dst_node);
 static void free_matrix_buffer_on_node(void *data_interface, unsigned node);
+static void cache_matrix_buffer_on_node(void *cached_interface, void *src_data_interface, unsigned node);
+static void reuse_matrix_buffer_on_node(void *dst_data_interface, const void *cached_interface, unsigned node);
 static size_t matrix_interface_get_size(starpu_data_handle_t handle);
 static size_t matrix_interface_get_alloc_size(starpu_data_handle_t handle);
 static uint32_t footprint_matrix_interface_crc32(starpu_data_handle_t handle);
@@ -37,6 +44,7 @@ static int matrix_compare(void *data_interface_a, void *data_interface_b);
 static int matrix_alloc_compare(void *data_interface_a, void *data_interface_b);
 static void display_matrix_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_matrix_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_matrix_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
@@ -46,8 +54,12 @@ struct starpu_data_interface_ops starpu_interface_matrix_ops =
 	.register_data_handle = register_matrix_handle,
 	.allocate_data_on_node = allocate_matrix_buffer_on_node,
 	.to_pointer = matrix_to_pointer,
-	.pointer_is_inside = matrix_pointer_is_inside,
 	.free_data_on_node = free_matrix_buffer_on_node,
+	.cache_data_on_node = cache_matrix_buffer_on_node,
+	.reuse_data_on_node = reuse_matrix_buffer_on_node,
+	.map_data = map_matrix,
+	.unmap_data = unmap_matrix,
+	.update_map = update_map_matrix,
 	.copy_methods = &matrix_copy_data_methods_s,
 	.get_size = matrix_interface_get_size,
 	.get_alloc_size = matrix_interface_get_alloc_size,
@@ -59,9 +71,13 @@ struct starpu_data_interface_ops starpu_interface_matrix_ops =
 	.interface_size = sizeof(struct starpu_matrix_interface),
 	.display = display_matrix_interface,
 	.pack_data = pack_matrix_handle,
+	.peek_data = peek_matrix_handle,
 	.unpack_data = unpack_matrix_handle,
 	.describe = describe,
-	.name = "STARPU_MATRIX_INTERFACE"
+	.name = "STARPU_MATRIX_INTERFACE",
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
 static void matrix_init(void *data_interface)
@@ -70,11 +86,11 @@ static void matrix_init(void *data_interface)
 	matrix_interface->allocsize = -1;
 }
 
-static void register_matrix_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_matrix_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_matrix_interface *matrix_interface = (struct starpu_matrix_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_matrix_interface *local_interface = (struct starpu_matrix_interface *)
@@ -83,8 +99,8 @@ static void register_matrix_handle(starpu_data_handle_t handle, unsigned home_no
 		if (node == home_node)
 		{
 			local_interface->ptr = matrix_interface->ptr;
-                        local_interface->dev_handle = matrix_interface->dev_handle;
-                        local_interface->offset = matrix_interface->offset;
+			local_interface->dev_handle = matrix_interface->dev_handle;
+			local_interface->offset = matrix_interface->offset;
 			local_interface->ld  = matrix_interface->ld;
 		}
 		else
@@ -111,25 +127,12 @@ static void *matrix_to_pointer(void *data_interface, unsigned node)
 	return (void*) matrix_interface->ptr;
 }
 
-static int matrix_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
-{
-	(void) node;
-	struct starpu_matrix_interface *matrix_interface = data_interface;
-	uint32_t ld = matrix_interface->ld;
-	uint32_t nx = matrix_interface->nx;
-	uint32_t ny = matrix_interface->ny;
-	size_t elemsize = matrix_interface->elemsize;
-
-	return (char*) ptr >= (char*) matrix_interface->ptr &&
-		(char*) ptr < (char*) matrix_interface->ptr + (ny-1)*ld*elemsize + nx*elemsize;
-}
-
-
 /* declare a new data with the matrix interface */
 void starpu_matrix_data_register_allocsize(starpu_data_handle_t *handleptr, int home_node,
 					   uintptr_t ptr, uint32_t ld, uint32_t nx,
 					   uint32_t ny, size_t elemsize, size_t allocsize)
 {
+	STARPU_ASSERT_MSG(ld >= nx, "ld = %u should not be less than nx = %u.", ld, nx);
 	struct starpu_matrix_interface matrix_interface =
 	{
 		.id = STARPU_MATRIX_INTERFACE_ID,
@@ -138,8 +141,8 @@ void starpu_matrix_data_register_allocsize(starpu_data_handle_t *handleptr, int
 		.nx = nx,
 		.ny = ny,
 		.elemsize = elemsize,
-                .dev_handle = ptr,
-                .offset = 0,
+		.dev_handle = ptr,
+		.offset = 0,
 		.allocsize = allocsize,
 	};
 #ifndef STARPU_SIMGRID
@@ -189,7 +192,7 @@ static int matrix_compare(void *data_interface_a, void *data_interface_b)
 	struct starpu_matrix_interface *matrix_a = (struct starpu_matrix_interface *) data_interface_a;
 	struct starpu_matrix_interface *matrix_b = (struct starpu_matrix_interface *) data_interface_b;
 
-	/* Two matricess are considered compatible if they have the same size */
+	/* Two matrices are considered compatible if they have the same size */
 	return (matrix_a->nx == matrix_b->nx)
 		&& (matrix_a->ny == matrix_b->ny)
 		&& (matrix_a->elemsize == matrix_b->elemsize);
@@ -200,7 +203,7 @@ static int matrix_alloc_compare(void *data_interface_a, void *data_interface_b)
 	struct starpu_matrix_interface *matrix_a = (struct starpu_matrix_interface *) data_interface_a;
 	struct starpu_matrix_interface *matrix_b = (struct starpu_matrix_interface *) data_interface_b;
 
-	/* Two matricess are considered allocation-compatible if they have the same size */
+	/* Two matrices are considered allocation-compatible if they have the same size */
 	return (matrix_a->allocsize == matrix_b->allocsize);
 }
 
@@ -214,6 +217,21 @@ static void display_matrix_interface(starpu_data_handle_t handle, FILE *f)
 
 #define IS_CONTIGUOUS_MATRIX(nx, ny, ld) ((nx) == (ld))
 
+//#define DYNAMIC_MATRICES
+
+struct pack_matrix_header
+{
+#ifdef DYNAMIC_MATRICES
+	/* Receiving matrices with different sizes from MPI */
+	/* FIXME: that would break alignment for O_DIRECT disk access...
+	 * while in the disk case, we do know the matrix size anyway */
+	/* FIXME: rather make MPI pack the data interface in the envelope for us? */
+	uint32_t nx;
+	uint32_t ny;
+	size_t elemsize;
+#endif
+};
+
 static int pack_matrix_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
@@ -226,14 +244,22 @@ static int pack_matrix_handle(starpu_data_handle_t handle, unsigned node, void *
 	uint32_t ny = matrix_interface->ny;
 	size_t elemsize = matrix_interface->elemsize;
 
-	*count = nx*ny*elemsize;
+	*count = nx*ny*elemsize + sizeof(struct pack_matrix_header);
 
 	if (ptr != NULL)
 	{
 		char *matrix = (void *)matrix_interface->ptr;
 
 		*ptr = (void *)starpu_malloc_on_node_flags(node, *count, 0);
-		char *cur = *ptr;
+
+		struct pack_matrix_header *header = *ptr;
+#ifdef DYNAMIC_MATRICES
+		header->nx = nx;
+		header->ny = ny;
+		header->elemsize = elemsize;
+#endif
+
+		char *cur = (char*) *ptr + sizeof(*header);
 
 		if (IS_CONTIGUOUS_MATRIX(nx, ny, ld))
 			memcpy(cur, matrix, nx*ny*elemsize);
@@ -252,7 +278,7 @@ static int pack_matrix_handle(starpu_data_handle_t handle, unsigned node, void *
 	return 0;
 }
 
-static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_matrix_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -264,7 +290,45 @@ static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void
 	uint32_t ny = matrix_interface->ny;
 	size_t elemsize = matrix_interface->elemsize;
 
-	STARPU_ASSERT(count == elemsize * nx * ny);
+	struct pack_matrix_header *header = ptr;
+
+#ifdef DYNAMIC_MATRICES
+	STARPU_ASSERT(count >= sizeof(*header));
+
+	if (IS_CONTIGUOUS_MATRIX(nx, ny, ld))
+	{
+		/* We can store whatever can fit */
+
+		STARPU_ASSERT_MSG(header->elemsize == elemsize,
+				"Data element size %u needs to be same as the received data element size %u",
+				(unsigned) elemsize, (unsigned) header->elemsize);
+
+		STARPU_ASSERT_MSG(header->nx * header->ny * header->elemsize <= matrix_interface->allocsize,
+				"Initial size of data %lu needs to be big enough for received data %ux%ux%u",
+				(unsigned long) matrix_interface->allocsize,
+				(unsigned) header->nx, (unsigned) header->ny,
+				(unsigned) header->elemsize);
+
+		/* Better keep it contiguous */
+		matrix_interface->ld = ld = header->nx;
+	}
+	else
+	{
+		STARPU_ASSERT_MSG(header->nx <= nx,
+				"Initial nx %u of data needs to be big enough for received data nx %u\n",
+				nx, header->nx);
+		STARPU_ASSERT_MSG(header->ny <= ny,
+				"Initial ny %u of data needs to be big enough for received data ny %u\n",
+				ny, header->ny);
+	}
+
+	matrix_interface->nx = nx = header->nx;
+	matrix_interface->ny = ny = header->ny;
+#endif
+
+	char *cur = (char*) ptr + sizeof(*header);
+
+	STARPU_ASSERT(count == sizeof(*header) + elemsize * nx * ny);
 
 	char *matrix = (void *)matrix_interface->ptr;
 
@@ -273,7 +337,6 @@ static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void
 	else
 	{
 		uint32_t y;
-		char *cur = ptr;
 		for(y=0 ; y<ny ; y++)
 		{
 			memcpy(matrix, cur, nx*elemsize);
@@ -282,6 +345,12 @@ static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void
 		}
 	}
 
+	return 0;
+}
+
+static int unpack_matrix_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_matrix_handle(handle, node, ptr, count);
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 
 	return 0;
@@ -430,6 +499,75 @@ static void free_matrix_buffer_on_node(void *data_interface, unsigned node)
 	struct starpu_matrix_interface *matrix_interface = (struct starpu_matrix_interface *) data_interface;
 
 	starpu_free_on_node(node, matrix_interface->dev_handle, matrix_interface->allocsize);
+	matrix_interface->ptr = 0;
+	matrix_interface->dev_handle = 0;
+}
+
+static void cache_matrix_buffer_on_node(void *cached_interface, void *src_data_interface, unsigned node STARPU_ATTRIBUTE_UNUSED)
+{
+	struct starpu_matrix_interface *cached_matrix_interface = cached_interface;
+	struct starpu_matrix_interface *src_matrix_interface = src_data_interface;
+
+	cached_matrix_interface->ptr = src_matrix_interface->ptr;
+	src_matrix_interface->ptr = 0;
+	cached_matrix_interface->dev_handle = src_matrix_interface->dev_handle;
+	src_matrix_interface->dev_handle = 0;
+	cached_matrix_interface->allocsize = src_matrix_interface->allocsize;
+	STARPU_ASSERT(src_matrix_interface->offset == 0);
+}
+
+static void reuse_matrix_buffer_on_node(void *dst_data_interface, const void *cached_interface, unsigned node STARPU_ATTRIBUTE_UNUSED)
+{
+	struct starpu_matrix_interface *dst_matrix_interface = dst_data_interface;
+	const struct starpu_matrix_interface *cached_matrix_interface = cached_interface;
+
+	dst_matrix_interface->ptr = cached_matrix_interface->ptr;
+	dst_matrix_interface->dev_handle = cached_matrix_interface->dev_handle;
+	dst_matrix_interface->offset = 0;
+	dst_matrix_interface->ld = dst_matrix_interface->nx; // by default
+}
+
+static int map_matrix(void *src_interface, unsigned src_node,
+		      void *dst_interface, unsigned dst_node)
+{
+	struct starpu_matrix_interface *src_matrix = src_interface;
+	struct starpu_matrix_interface *dst_matrix = dst_interface;
+	int ret;
+	uintptr_t mapped;
+
+	/* map area ld*(ny-1)+nx */
+	mapped = starpu_interface_map(src_matrix->dev_handle, src_matrix->offset, src_node, dst_node, (src_matrix->ld*(src_matrix->ny-1)+src_matrix->nx)*src_matrix->elemsize, &ret);
+	if (mapped)
+	{
+		dst_matrix->dev_handle = mapped;
+		dst_matrix->offset = 0;
+		if (starpu_node_get_kind(dst_node) != STARPU_OPENCL_RAM)
+			dst_matrix->ptr = mapped;
+		dst_matrix->ld = src_matrix->ld;
+		return 0;
+	}
+	return ret;
+}
+
+static int unmap_matrix(void *src_interface, unsigned src_node,
+			void *dst_interface, unsigned dst_node)
+{
+	struct starpu_matrix_interface *src_matrix = src_interface;
+	struct starpu_matrix_interface *dst_matrix = dst_interface;
+
+	int ret = starpu_interface_unmap(src_matrix->dev_handle, src_matrix->offset, src_node, dst_matrix->dev_handle, dst_node, (src_matrix->ld*(src_matrix->ny-1)+src_matrix->nx)*src_matrix->elemsize);
+	dst_matrix->dev_handle = 0;
+
+	return ret;
+}
+
+static int update_map_matrix(void *src_interface, unsigned src_node,
+			     void *dst_interface, unsigned dst_node)
+{
+	struct starpu_matrix_interface *src_matrix = src_interface;
+	struct starpu_matrix_interface *dst_matrix = dst_interface;
+
+	return starpu_interface_update_map(src_matrix->dev_handle, src_matrix->offset, src_node, dst_matrix->dev_handle, dst_matrix->offset, dst_node, (src_matrix->ld*(src_matrix->ny-1)+src_matrix->nx)*src_matrix->elemsize);
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
