@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016,2017                      Inria
- * Copyright (C) 2017,2018                                Federal University of Rio Grande do Sul (UFRGS)
- * Copyright (C) 2012,2014,2017,2018                      CNRS
- * Copyright (C) 2010-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2017-2019  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,10 +24,14 @@
 #ifdef STARPU_USE_FXT
 
 #ifdef STARPU_HAVE_POTI
+int _starpu_poti_JobState;
+int _starpu_poti_checkPointState;
 #ifdef HAVE_POTI_INIT_CUSTOM
 int _starpu_poti_extendedSetState = -1;
 int _starpu_poti_semiExtendedSetState = -1;
 int _starpu_poti_MemoryEvent = -1;
+int _starpu_poti_CommLinkStart = -1;
+int _starpu_poti_MpiLinkStart = -1;
 #endif
 #endif
 
@@ -52,7 +54,8 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 						     "Y string",
 						     /* "Z string", */
 						     "Iteration string",
-						     "Subiteration string");
+						     "Subiteration string",
+						     "NumaNodes string");
 	_starpu_poti_semiExtendedSetState = poti_header_DeclareEvent (PAJE_SetState,
 						     6,
 						     "Size string",
@@ -63,6 +66,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 						     "SubmitOrder string"
 						     );
 #ifdef HAVE_POTI_USER_NEWEVENT
+	_starpu_poti_CommLinkStart = poti_header_DeclareEvent(PAJE_StartLink, 4, "Handle string", "HName string", "X string", "Y string");
 	if (options->memory_states)
 	{
 		_starpu_poti_MemoryEvent = poti_header_DeclareEvent (PAJE_NewEvent,
@@ -72,6 +76,9 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 							     "Size string",
 							     "Dest string");
 	}
+	_starpu_poti_MpiLinkStart = poti_header_DeclareEvent(PAJE_StartLink, 5, "MPITAG string", "MPIType string", "Priority string", "Handle string", "HName string", "X string", "Y string");
+	_starpu_poti_checkPointState = poti_header_DeclareEvent(PAJE_NewEvent, 2, "CheckpointInstance string", "CheckpointDomain string");
+	_starpu_poti_JobState = poti_header_DeclareEvent(PAJE_SetState, 1, "JobId string");
 #endif
 #else
 	poti_header(1,1);
@@ -86,6 +93,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	fprintf(file, "%%	Alias	string\n");
 	fprintf(file, "%%	Type	string\n");
 	fprintf(file, "%%	Name	string\n");
+	fprintf(file, "%%	Color	color\n");
 	fprintf(file, "%%EndEventDef\n");
 	fprintf(file, "%%EventDef	PajeDefineStateType	3\n");
 	fprintf(file, "%%	Alias	string\n");
@@ -196,6 +204,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	/* fprintf(file, "%%	Z	string\n"); */
 	fprintf(file, "%%	Iteration	string\n");
 	fprintf(file, "%%	Subiteration	string\n");
+	fprintf(file, "%%	NumaNodes	string\n");
 	fprintf(file, "%%EndEventDef\n");
 	fprintf(file, "%%EventDef PajeSetState 21\n");
 	fprintf(file, "%%	Time	date\n");
@@ -222,6 +231,48 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 		fprintf(file, "%%	Tid	string\n");
 		fprintf(file, "%%EndEventDef\n");
 	}
+	fprintf(file, "%%EventDef	PajeStartLink	23\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	StartContainer	string\n");
+	fprintf(file, "%%	Key	string\n");
+	fprintf(file, "%%	MPITAG	string\n");
+	fprintf(file, "%%	MPIType	string\n");
+	fprintf(file, "%%	Priority	string\n");
+	fprintf(file, "%%	Handle	string\n");
+	fprintf(file, "%%	HName	string\n");
+	fprintf(file, "%%	X	string\n");
+	fprintf(file, "%%	Y	string\n");
+	fprintf(file, "%%EndEventDef\n");
+	fprintf(file, "%%EventDef	PajeStartLink	24\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	StartContainer	string\n");
+	fprintf(file, "%%	Key	string\n");
+	fprintf(file, "%%	Handle	string\n");
+	fprintf(file, "%%	HName	string\n");
+	fprintf(file, "%%	X	string\n");
+	fprintf(file, "%%	Y	string\n");
+	fprintf(file, "%%EndEventDef\n");
+	fprintf(file, "%%EventDef	PajeNewEvent	25\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	CheckpointInstance	string\n");
+	fprintf(file, "%%	CheckpointDomain	string\n");
+	fprintf(file, "%%EndEventDef\n");
+	fprintf(file, "%%EventDef	PajeSetState	26\n");
+	fprintf(file, "%%	Time	date\n");
+	fprintf(file, "%%	Container	string\n");
+	fprintf(file, "%%	Type	string\n");
+	fprintf(file, "%%	Value	string\n");
+	fprintf(file, "%%	JobId	string\n");
+	fprintf(file, "%%EndEventDef\n");
 #endif
 
 #ifdef STARPU_HAVE_POTI
@@ -235,8 +286,8 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	poti_DefineContainerType("MPICt", "P", "MPI Communication Thread");
 	poti_DefineContainerType("Sc", "P", "Scheduler");
 	poti_DefineEventType("prog_event", "P", "program event type");
-       poti_DefineEventType("pu", "P", "task push");
-       poti_DefineEventType("po", "P", "task pop");
+	poti_DefineEventType("pu", "P", "task push");
+	poti_DefineEventType("po", "P", "task pop");
 	poti_DefineEventType("register", "P", "data registration");
 	poti_DefineEventType("unregister", "P", "data unregistration");
 
@@ -246,7 +297,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	poti_DefineEventType("SO", "Mm", "data state owner");
 	poti_DefineEventType("WU", "Mm", "data wont use");
 	poti_DefineEventType("Al", "Mm", "Allocating Start");
-       poti_DefineEventType("rc", "Mm", "Request Created");
+	poti_DefineEventType("rc", "Mm", "Request Created");
 	poti_DefineEventType("AlE", "Mm", "Allocating End");
 	poti_DefineEventType("Alr", "Mm", "Allocating Async Start");
 	poti_DefineEventType("AlrE", "Mm", "Allocating Async End");
@@ -275,7 +326,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	/* Types for the Worker of the Memory Node */
 	poti_DefineEventType("user_event", "P", "user event type");
 	poti_DefineEventType("thread_event", "T", "thread event type");
-	poti_DefineVariableType("gf", "W", "GFlops", "0 0 0");
+	poti_DefineVariableType("gf", "W", "GFlop/s", "0 0 0");
 	poti_DefineStateType("S", "T", "Thread State");
 	poti_DefineEntityValue("I", "S", "Idle", ".9 .1 0");
 	poti_DefineEntityValue("In", "S", "Initializing", "0.0 .7 1.0");
@@ -284,6 +335,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	poti_DefineEntityValue("Po", "S", "PushingOutput", "0.1 1.0 1.0");
 	poti_DefineEntityValue("C", "S", "Callback", ".0 .3 .8");
 	poti_DefineEntityValue("B", "S", "Overhead", ".5 .18 .0");
+	poti_DefineEntityValue("Ps", "S", "Parallel sync", ".5 .18 1.0");
 	poti_DefineEntityValue("E", "S", "Executing", ".0 .6 .5");
 	poti_DefineEntityValue("Sc", "S", "Scheduling", ".7 .36 .0");
 	poti_DefineEntityValue("Sl", "S", "Sleeping", ".9 .1 .0");
@@ -304,6 +356,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	poti_DefineEntityValue("Po", "WS", "PushingOutput", "0.1 1.0 1.0");
 	poti_DefineEntityValue("C", "WS", "Callback", ".0 .3 .8");
 	poti_DefineEntityValue("B", "WS", "Overhead", ".5 .18 .0");
+	poti_DefineEntityValue("Ps", "WS", "Parallel sync", ".5 .18 1.0");
 	poti_DefineEntityValue("E", "WS", "Executing", ".0 .6 .5");
 	poti_DefineEntityValue("Sc", "WS", "Scheduling", ".7 .36 .0");
 	poti_DefineEntityValue("Sl", "WS", "Sleeping", ".9 .1 .0");
@@ -343,6 +396,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	poti_DefineEntityValue("Bu", "US", "Building task", ".5 .18 .0");
 	poti_DefineEntityValue("Su", "US", "Submitting task", ".3 .09 .0");
 	poti_DefineEntityValue("C", "US", "Callback", ".0 .3 .8");
+	poti_DefineEntityValue("Sc", "US", "Scheduling", ".7 .36 .0");
 	poti_DefineEntityValue("Th", "US", "Throttling task submission", ".8 .6 .6");
 	poti_DefineEntityValue("MD", "US", "Decoding task for MPI", ".5 .18 .2");
 	poti_DefineEntityValue("MPr", "US", "Preparing task for MPI", ".4 .14 .2");
@@ -364,6 +418,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 		poti_DefineEntityValue("Po", ctx, "PushingOutput", "0.1 1.0 1.0");
 		poti_DefineEntityValue("C", ctx, "Callback", ".0 .3 .8");
 		poti_DefineEntityValue("B", ctx, "Overhead", ".5 .18 .0");
+		poti_DefineEntityValue("Ps", ctx, "Parallel sync", ".5 .18 1.0");
 		poti_DefineEntityValue("E", ctx, "Executing", ".0 .6 .5");
 		poti_DefineEntityValue("Sc", ctx, "Scheduling", ".7 .36 .0");
 		poti_DefineEntityValue("Sl", ctx, "Sleeping", ".9 .1 .0");
@@ -375,11 +430,12 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 	/* Types for the Scheduler */
 	poti_DefineVariableType("nsubmitted", "Sc", "Number of Submitted Uncompleted Tasks", "0 0 0");
 	poti_DefineVariableType("nready", "Sc", "Number of Ready Tasks", "0 0 0");
-	poti_DefineVariableType("gft", "Sc", "Total GFlops", "0 0 0");
+	poti_DefineVariableType("gft", "Sc", "Total GFlop/s", "0 0 0");
 
 	/* Link types */
 	poti_DefineLinkType("MPIL", "MPIP", "MPICt", "MPICt", "MPI communication");
 	poti_DefineLinkType("F", "P", "Mm", "Mm", "Intra-node data Fetch");
+	poti_DefineLinkType("TF", "P", "Mm", "Mm", "Intra-node data TaskPreFetch");
 	poti_DefineLinkType("PF", "P", "Mm", "Mm", "Intra-node data PreFetch");
 	poti_DefineLinkType("IF", "P", "Mm", "Mm", "Intra-node data IdleFetch");
 	poti_DefineLinkType("WSL", "P", "W", "W", "Work steal");
@@ -397,48 +453,50 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 1       W      T       \"Worker\"                               \n\
 1       MPICt   P       \"MPI Communication Thread\"              \n\
 1       Sc       P       \"Scheduler State\"                        \n\
-2       prog_event   P       \"program event type\"				\n\
-2       pu   P       \"Task Push\"                             \n\
-2       po   P       \"Task Pop\"                              \n\
-2       register     P       \"data registration\"				\n\
-2       unregister     P       \"data unregistration\"				\n\
-2       user_event   P       \"user event type\"				\n\
-2       thread_event   T       \"thread event type\"				\n\
-2       user_user_event   UT       \"user event type\"				\n\
-2       user_thread_event   UT       \"thread event type\"				\n\
-2       MPIev   MPICt    \"MPI event type\"			\n\
+2       prog_event   P       \"program event type\"		\"1.0 1.0 1.0\" 		\n\
+2       checkpoint_begin  P       \"Checkpoint Begin\"	\"0.0 1.0 0.0\" 			\n\
+2       checkpoint_end  P       \"Checkpoint End\"		\"0.0 0.0 1.0\" 		\n\
+2       pu   P       \"Task Push\"           \"1.0 1.0 1.0\"                   \n\
+2       po   P       \"Task Pop\"          \"1.0 1.0 1.0\"                     \n\
+2       register     P       \"data registration\"		\"1.0 1.0 1.0\" 		\n\
+2       unregister     P       \"data unregistration\"	\"1.0 1.0 1.0\" 			\n\
+2       user_event   P       \"user event type\"		\"1.0 1.0 1.0\" 		\n\
+2       thread_event   T       \"thread event type\"	\"1.0 1.0 1.0\" 			\n\
+2       user_user_event   UT       \"user event type\"		\"1.0 1.0 1.0\" 		\n\
+2       user_thread_event   UT       \"thread event type\"	\"1.0 1.0 1.0\" 			\n\
+2       MPIev   MPICt    \"MPI event type\"		\"1.0 1.0 1.0\" 	\n\
 3       S       T       \"Thread State\"                        \n\
 3       CtS     MPICt    \"Communication Thread State\"          \n");
 	for (i=1; i<STARPU_NMAX_SCHED_CTXS; i++)
 		fprintf(file, "3       Ctx%u      W     \"InCtx%u\"         		\n", i, i);
 	fprintf(file, "\
-2       SI       Mm \"data state invalid\"                            \n\
-2       SS       Mm \"data state shared\"                            \n\
-2       SO       Mm \"data state owner\"                            \n\
-2       WU       Mm \"data wont use\"                            \n\
-2       Al       Mm    \"Allocating Start\"    \n\
-2       rc       Mm    \"Request Created\"    \n\
-2       AlE      Mm    \"Allocating End\"    \n\
-2       Alr      Mm    \"Allocating Async Start\"    \n\
-2       AlrE     Mm    \"Allocating Async End\"    \n\
-2       Fe       Mm    \"Free Start\"    \n\
-2       FeE      Mm    \"Free End\"    \n\
-2       Wb       Mm    \"WritingBack Start\"    \n\
-2       WbE      Mm    \"WritingBack End\"    \n\
-2       DCo      Mm    \"DriverCopy Start\"    \n\
-2       DCoE     Mm    \"DriverCopy End\"    \n\
-2       DCoA     Mm    \"DriverCopyAsync Start\"    \n\
-2       DCoAE    Mm    \"DriverCopyAsync End\"    \n\
+2       SI       Mm \"data state invalid\"            \"1.0 1.0 1.0\"                 \n\
+2       SS       Mm \"data state shared\"        \"1.0 1.0 1.0\"                    \n\
+2       SO       Mm \"data state owner\"        \"1.0 1.0 1.0\"                     \n\
+2       WU       Mm \"data wont use\"           \"1.0 1.0 1.0\"                  \n\
+2       Al       Mm    \"Allocating Start\"  \"1.0 1.0 1.0\"   \n\
+2       rc       Mm    \"Request Created\"  \"1.0 1.0 1.0\"   \n\
+2       AlE      Mm    \"Allocating End\" \"1.0 1.0 1.0\"    \n\
+2       Alr      Mm    \"Allocating Async Start\"  \"1.0 1.0 1.0\"   \n\
+2       AlrE     Mm    \"Allocating Async End\" \"1.0 1.0 1.0\"    \n\
+2       Fe       Mm    \"Free Start\" \"1.0 1.0 1.0\"    \n\
+2       FeE      Mm    \"Free End\"   \"1.0 1.0 1.0\"  \n\
+2       Wb       Mm    \"WritingBack Start\" \"1.0 1.0 1.0\"    \n\
+2       WbE      Mm    \"WritingBack End\"  \"1.0 1.0 1.0\"   \n\
+2       DCo      Mm    \"DriverCopy Start\"  \"1.0 1.0 1.0\"   \n\
+2       DCoE     Mm    \"DriverCopy End\" \"1.0 1.0 1.0\"    \n\
+2       DCoA     Mm    \"DriverCopyAsync Start\"  \"1.0 1.0 1.0\"   \n\
+2       DCoAE    Mm    \"DriverCopyAsync End\"  \"1.0 1.0 1.0\"   \n\
 3       MS       Mm       \"Memory Node State\"                        \n\
 4       nsubmitted    Sc       \"Number of Submitted Uncompleted Tasks\"                        \n\
 4       nready    Sc       \"Number of Ready Tasks\"                        \n\
-4       gft    Sc       \"Total GFlops\"                        \n\
+4       gft    Sc       \"Total GFlop/s\"                        \n\
 4       use     Mm       \"Used (MB)\"                        \n\
 4       bwi_mm     Mm       \"Bandwidth In (MB/s)\"                        \n\
 4       bwo_mm     Mm       \"Bandwidth Out (MB/s)\"                        \n\
 4       bwi_mpi     MPICt       \"Bandwidth In (MB/s)\"                        \n\
 4       bwo_mpi     MPICt       \"Bandwidth Out (MB/s)\"                        \n\
-4       gf      W       \"GFlops\"                        \n\
+4       gf      W       \"GFlop/s\"                        \n\
 6       I       S       Idle         \".9 .1 .0\"		\n\
 6       In       S      Initializing       \"0.0 .7 1.0\"            \n\
 6       D       S      Deinitializing       \"0.0 .1 .7\"            \n\
@@ -446,6 +504,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 6       Po       S      PushingOutput       \"0.1 1.0 1.0\"            \n\
 6       C       S       Callback       \".0 .3 .8\"            \n\
 6       B       S       Overhead         \".5 .18 .0\"		\n\
+6       Ps      S       \"Parallel sync\"  \".5 .18 1.0\"		\n\
 6       E       S       Executing         \".0 .6 .5\"		\n\
 6       Sc       S      Scheduling         \".7 .36 .0\"		\n\
 6       Sl       S      Sleeping         \".9 .1 .0\"		\n\
@@ -466,6 +525,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 6       Po       WS      PushingOutput       \"0.1 1.0 1.0\"            \n\
 6       C       WS       Callback       \".0 .3 .8\"            \n\
 6       B       WS       Overhead         \".5 .18 .0\"		\n\
+6       Ps      WS       \"Parallel sync\"  \".5 .18 1.0\"		\n\
 6       E       WS       Executing         \".0 .6 .5\"		\n\
 6       Sc       WS      Scheduling         \".7 .36 .0\"		\n\
 6       Sl       WS      Sleeping         \".9 .1 .0\"		\n\
@@ -479,6 +539,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 6       Bu      US      \"Building task\"   \".5 .18 .0\"		\n\
 6       Su      US      \"Submitting task\" \".3 .09 .0\"		\n\
 6       C       US      \"Callback\" \".0 .3 .8\"		\n\
+6       Sc      US      \"Scheduling\"         \".7 .36 .0\"		\n\
 6       Th      US      \"Throttling task submission\" \".8 .6 .6\"		\n\
 6       MD      US      \"Decoding task for MPI\" \".5 .18 .2\"		\n\
 6       MPr     US      \"Preparing task for MPI\" \".4 .14 .2\"		\n\
@@ -514,13 +575,14 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 6       Po       Ctx%u      PushingOutput       \"0.1 1.0 1.0\"            \n\
 6       C       Ctx%u       Callback       \".0 .3 .8\"            \n\
 6       B       Ctx%u       Overhead         \".5 .18 .0\"		\n\
+6       Ps      Ctx%u       \"Parallel sync\"  \".5 .18 1.0\"		\n\
 6       E       Ctx%u       Executing         \".0 .6 .5\"		\n\
 6       Sc       Ctx%u      Scheduling         \".7 .36 .0\"		\n\
 6       Sl       Ctx%u      Sleeping         \".9 .1 .0\"		\n\
 6       P       Ctx%u       Progressing         \".1 .3 .1\"		\n\
 6       U       Ctx%u       Unpartitioning         \".0 .0 1.0\"	\n\
 6       H       Ctx%u       Hypervisor         \".5 .18 .0\"		\n",
-		i, i, i, i, i, i, i, i, i, i, i, i, i);
+		i, i, i, i, i, i, i, i, i, i, i, i, i, i);
 	fprintf(file, "\
 6       A       MS      Allocating         \".4 .1 .0\"		\n\
 6       Ar       MS      AllocatingReuse       \".1 .1 .8\"		\n\
@@ -533,6 +595,7 @@ void _starpu_fxt_write_paje_header(FILE *file STARPU_ATTRIBUTE_UNUSED, struct st
 6       No       MS     Nothing         \".0 .0 .0\"		\n\
 5       MPIL     MPIP	MPICt	MPICt   \"MPI communication\"\n\
 5       F       P	Mm	Mm      \"Intra-node data Fetch\"\n\
+5       TF      P	Mm	Mm      \"Intra-node data TaskPreFetch\"\n\
 5       PF      P	Mm	Mm      \"Intra-node data PreFetch\"\n\
 5       IF      P	Mm	Mm      \"Intra-node data IdleFetch\"\n\
 5       WSL     P	W	W       \"Work steal\"\n");
