@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -40,7 +40,7 @@
  * tasks by calling can_push to their parent (classically, just after a successful pop have
  * been made by its associated worker_component), this call goes up to the window_component which
  * pops a task from its local queue and try to schedule it by pushing it to the
- * decision_component. 
+ * decision_component.
  * Finally, the task will be pushed to the prio_component which is the direct
  * parent in the tree of the worker_component the task has been scheduled on. This
  * component will push the task on its local queue if no one of the two thresholds
@@ -57,6 +57,7 @@ static void initialize_heft_center_policy(unsigned sched_ctx_id)
 			STARPU_SCHED_SIMPLE_FIFOS_BELOW |
 			STARPU_SCHED_SIMPLE_FIFOS_BELOW_PRIO |
 			STARPU_SCHED_SIMPLE_FIFOS_BELOW_READY |
+			STARPU_SCHED_SIMPLE_FIFOS_BELOW_EXP |
 			STARPU_SCHED_SIMPLE_IMPL, sched_ctx_id);
 }
 
@@ -70,8 +71,8 @@ struct starpu_sched_policy _starpu_sched_modular_heft_policy =
 	.pop_task = starpu_sched_tree_pop_task,
 	.pre_exec_hook = starpu_sched_component_worker_pre_exec_hook,
 	.post_exec_hook = starpu_sched_component_worker_post_exec_hook,
-	.pop_every_task = NULL,
 	.policy_name = "modular-heft",
 	.policy_description = "heft modular policy",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
