@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,6 +36,9 @@ extern void test_block_opencl_func(void *buffers[], void *args);
 static starpu_data_handle_t _block_handle;
 static starpu_data_handle_t _block2_handle;
 
+static int _block[NX*NY*NZ];
+static int _block2[NX*NY*NZ];
+
 struct test_config block_config =
 {
 	.cpu_func      = test_block_cpu_func,
@@ -44,19 +47,15 @@ struct test_config block_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_block_opencl_func,
-#endif
-#ifdef STARPU_USE_MIC
-	.cpu_func_name = "test_block_cpu_func",
 #endif
 	.handle        = &_block_handle,
+	.ptr           = _block,
 	.dummy_handle  = &_block2_handle,
+	.dummy_ptr     = _block2,
 	.copy_failed   = SUCCESS,
 	.name          = "block_interface"
 };
 
-static int _block[NX*NY*NZ];
-static int _block2[NX*NY*NZ];
-
 static void
 register_data(void)
 {
@@ -139,24 +138,21 @@ main(int argc, char **argv)
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
 	conf.nopencl = 1;
-	conf.nmic = -1;
 
-	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
-		goto enodev;
+	int ret = starpu_initialize(&conf, &argc, &argv);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	register_data();
 
-	run_tests(&block_config, &summary);
+	ret = run_tests(&block_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, &summary);
+	if (ret) data_interface_test_summary_print(stderr, &summary);
 
 	return data_interface_test_summary_success(&summary);
-
-enodev:
-	return STARPU_TEST_SKIPPED;
 }
 
