@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012-2013,2015-2017                      CNRS
- * Copyright (C) 2010-2014,2016                           Université de Bordeaux
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -37,8 +35,9 @@ static starpu_data_handle_t v_handle;
  */
 
 #ifdef STARPU_USE_CUDA
-static void cuda_memset_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void cuda_memset_codelet(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -64,20 +63,23 @@ static void opencl_memset_codelet(void *buffers[], void *args)
 	STARPU_ASSERT(v != NULL);
 	memset(v, 42, length);
 
-	clEnqueueWriteBuffer(queue,
-			     buffer,
-			     CL_FALSE,
-			     0,      /* offset */
-			     length, /* sizeof (char) */
-			     v,
-			     0,      /* num_events_in_wait_list */
-			     NULL,   /* event_wait_list */
-			     NULL    /* event */);
+	cl_int err;
+	err = clEnqueueWriteBuffer(queue,
+				   buffer,
+				   CL_FALSE,
+				   0,      /* offset */
+				   length, /* sizeof (char) */
+				   v,
+				   0,      /* num_events_in_wait_list */
+				   NULL,   /* event_wait_list */
+				   NULL    /* event */);
+	if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 }
 #endif
 
-void cpu_memset_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_memset_codelet(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -106,8 +108,9 @@ static struct starpu_codelet memset_cl =
  *	Check content
  */
 
-void cpu_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_check_content_codelet(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -125,8 +128,9 @@ void cpu_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_arg
 }
 
 #ifdef STARPU_USE_CUDA
-static void cuda_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void cuda_check_content_codelet(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -149,6 +153,8 @@ static void cuda_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED vo
 #ifdef STARPU_USE_OPENCL
 static void opencl_check_content_codelet(void *buffers[], void *args)
 {
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_command_queue queue;
@@ -163,15 +169,19 @@ static void opencl_check_content_codelet(void *buffers[], void *args)
 	for (i = 0; i < length; i++)
 	{
 		char dst;
-		clEnqueueReadBuffer(queue,
-				    buf,
-				    CL_FALSE,
-				    i * sizeof(dst),
-				    sizeof(dst),
-				    &dst,
-				    0,      /* num_events_in_wait_list */
-				    NULL,   /* event_wait_list */
-				    NULL    /* event */);
+		cl_int err;
+
+		err = clEnqueueReadBuffer(queue,
+					  buf,
+					  CL_FALSE,
+					  i * sizeof(dst),
+					  sizeof(dst),
+					  &dst,
+					  0,      /* num_events_in_wait_list */
+					  NULL,   /* event_wait_list */
+					  NULL    /* event */);
+		if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
+
 		clFinish(queue);
 		if (dst != 42)
 		{
