@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2017-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2017-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,8 +36,8 @@
 #define N 5
 #define LIMIT "60"
 #else
-#define N 20
-#define LIMIT "1000"
+#define N 10
+#define LIMIT "250"
 #endif
 
 /* Define the interface */
@@ -48,6 +48,12 @@ int main(int argc, char **argv)
 {
 	return STARPU_TEST_SKIPPED;
 }
+#elif STARPU_MAXNODES == 1
+/* Cannot register a disk */
+int main(int argc, char **argv)
+{
+	return STARPU_TEST_SKIPPED;
+}
 #else
 
 /* Sample Data interface with variable size */
@@ -65,10 +71,10 @@ struct variable_size_interface
 
 static struct starpu_data_interface_ops starpu_interface_variable_size_ops;
 
-static void register_variable_size(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_variable_size(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct variable_size_interface *variable_size_interface = data_interface;
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct variable_size_interface *local_interface =
@@ -86,11 +92,7 @@ static void register_variable_size(starpu_data_handle_t handle, unsigned home_no
 
 void variable_size_data_register(starpu_data_handle_t *handleptr, unsigned x, unsigned y)
 {
-	if (starpu_interface_variable_size_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
-	{
-		starpu_interface_variable_size_ops.interfaceid = starpu_data_interface_get_next_id();
-	}
-	struct variable_size_interface interface =
+	struct variable_size_interface vsinterface =
 	{
 		.id = starpu_interface_variable_size_ops.interfaceid,
 		.x = x,
@@ -98,20 +100,25 @@ void variable_size_data_register(starpu_data_handle_t *handleptr, unsigned x, un
 	};
 
 	/* Simulate that tiles close to the diagonal are more dense */
-	interface.size = FULLSIZE * (starpu_lrand48() % 1024 + 1024) / 2048. * (N-sqrt(abs((int)x-(int)y)*N)) / N;
+	vsinterface.size = FULLSIZE * (starpu_lrand48() % 1024 + 1024) / 2048. * (N-sqrt(abs((int)x-(int)y)*N)) / N;
 	/* Round to page size */
-	interface.size -= interface.size & (65536-1);
+	vsinterface.size -= vsinterface.size & (65536-1);
 
-	_starpu_simgrid_data_new(interface.size);
+	_starpu_simgrid_data_new(vsinterface.size);
 
-	starpu_data_register(handleptr, -1, &interface, &starpu_interface_variable_size_ops);
+	starpu_data_register(handleptr, -1, &vsinterface, &starpu_interface_variable_size_ops);
 }
 
 static size_t variable_size_get_size(starpu_data_handle_t handle)
 {
-	struct variable_size_interface *interface =
-		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
-	return interface->size;
+	struct variable_size_interface *vsinterface = starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+	return vsinterface->size;
+}
+
+static size_t variable_size_get_max_size(starpu_data_handle_t handle)
+{
+	(void)handle;
+	return FULLSIZE;
 }
 
 static uint32_t variable_size_footprint(starpu_data_handle_t handle)
@@ -143,8 +150,7 @@ static starpu_ssize_t describe_variable_size(void *data_interface, char *buf, si
 }
 
 /* returns the size of the allocated area */
-static starpu_ssize_t allocate_variable_size_on_node(void *data_interface,
-						   unsigned dst_node)
+static starpu_ssize_t allocate_variable_size_on_node(void *data_interface, unsigned dst_node)
 {
 	struct variable_size_interface *variable_interface = data_interface;
 	variable_interface->ptr = starpu_malloc_on_node_flags(dst_node, variable_interface->size, STARPU_MALLOC_PINNED | STARPU_MALLOC_COUNT | STARPU_MEMORY_OVERFLOW);
@@ -154,13 +160,13 @@ static starpu_ssize_t allocate_variable_size_on_node(void *data_interface,
 	return 0;
 }
 
-static void free_variable_size_on_node(void *data_interface,
-					unsigned node)
+static void free_variable_size_on_node(void *data_interface, unsigned node)
 {
 	struct variable_size_interface *variable_interface = data_interface;
 	starpu_free_on_node(node, variable_interface->ptr, variable_interface->size);
 	if (node == STARPU_MAIN_RAM)
 		_starpu_simgrid_data_free(variable_interface->size);
+	variable_interface->ptr = 0;
 }
 
 static int variable_size_copy(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
@@ -193,12 +199,14 @@ static struct starpu_data_interface_ops starpu_interface_variable_size_ops =
 	.free_data_on_node = free_variable_size_on_node,
 	.copy_methods = &variable_size_copy_data_methods,
 	.get_size = variable_size_get_size,
+	.get_max_size = variable_size_get_max_size,
 	.footprint = variable_size_footprint,
 	.compare = variable_size_compare,
 	.interfaceid = STARPU_UNKNOWN_INTERFACE_ID,
 	.interface_size = sizeof(struct variable_size_interface),
 	.display = display_variable_size,
 	.pack_data = NULL,
+	.peek_data = NULL,
 	.unpack_data = NULL,
 	.describe = describe_variable_size,
 
@@ -207,7 +215,6 @@ static struct starpu_data_interface_ops starpu_interface_variable_size_ops =
 };
 
 
-
 static void kernel(void *descr[], void *cl_arg)
 {
 	struct variable_size_interface *variable_interface = descr[0];
@@ -298,7 +305,12 @@ int main(void)
 	setenv("STARPU_DISK_SWAP_BACKEND", "unistd", 0);
 #endif
 
-	ret = starpu_init(NULL);
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -308,8 +320,8 @@ int main(void)
 			variable_size_data_register(&handles[x][y], x, y);
 
 			ret = starpu_task_insert(&cl_init, STARPU_W, handles[x][y], 0);
-			if (ret == ENODEV)
-				goto enodev;
+			if (ret == -ENODEV) goto enodev;
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 #ifdef STARPU_SIMGRID
 			starpu_sleep(0.0005);
 #endif
@@ -321,7 +333,10 @@ int main(void)
 	for (i = 0; i < N; i++)
 		for (x = i; x < N; x++)
 			for (y = x; y < N; y++)
-				starpu_task_insert(&cl, STARPU_RW, handles[x][y], STARPU_PRIORITY, (2*N-x-y), 0);
+			{
+				ret = starpu_task_insert(&cl, STARPU_RW, handles[x][y], STARPU_PRIORITY, (2*N-x-y), 0);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+			}
 
 	starpu_task_wait_for_all();
 
