@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013,2015-2017                      CNRS
- * Copyright (C) 2010-2014,2016,2017                      Université de Bordeaux
- * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -30,7 +28,7 @@ void dummy_func(void *descr[], void *arg)
 {
 	unsigned duration = (uintptr_t) arg;
 	if (duration)
-		usleep(duration);
+		starpu_usleep(duration);
 }
 
 static struct starpu_codelet dummy_Rcodelet =
@@ -78,17 +76,24 @@ static struct starpu_task *create_dummy_task(int write, int data, unsigned durat
 int main(void)
 {
 	int ret;
-	/* We have 27 toggles to try below, thus 2^27 possibilities */
+	/* We have 17 toggles to try below, thus 2^17 possibilities */
 	unsigned loop, nloops = 128*1024;
-	unsigned duration = 1000;
+	unsigned duration = 100;
 
 	starpu_data_handle_t handle1, handle2;
 
 #ifdef STARPU_QUICK_CHECK
 	return STARPU_TEST_SKIPPED;
 #endif
+	struct starpu_conf conf;
 
-	ret = starpu_init(NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -224,23 +229,23 @@ int main(void)
 
 		ret = starpu_task_submit(taskA);
 		if (ret == -ENODEV)
-			return STARPU_TEST_SKIPPED;
+			goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
 		if (loop & 32768)
-			usleep(duration);
+			starpu_usleep(duration);
 
 		ret = starpu_task_submit(taskB);
 		if (ret == -ENODEV)
-			return STARPU_TEST_SKIPPED;
+			goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
 		if (loop & 65536)
-			usleep(duration);
+			starpu_usleep(duration);
 
 		ret = starpu_task_submit(taskC);
 		if (ret == -ENODEV)
-			return STARPU_TEST_SKIPPED;
+			goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
 		ret = starpu_task_wait(taskA);
@@ -254,9 +259,10 @@ int main(void)
 	}
 	while(0);
 
+enodev:
 	starpu_data_unregister(handle1);
 	starpu_data_unregister(handle2);
 	starpu_shutdown();
 
-	return EXIT_SUCCESS;
+	return ret == -ENODEV ? STARPU_TEST_SKIPPED : EXIT_SUCCESS;
 }
