@@ -1,7 +1,7 @@
-#!/bin/sh
+#!/bin/bash
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+# Copyright (C) 2016-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -14,6 +14,9 @@
 #
 # See the GNU Lesser General Public License in COPYING.LGPL for more details.
 #
+
+if test -n "$STARPU_MICROBENCHS_DISABLED" ; then exit 77 ; fi
+
 . $(dirname $0)/microbench.sh
 
 XFAIL="heteroprio"
@@ -25,4 +28,6 @@ fi
 
 _STARPU_LAUNCH="$STARPU_LAUNCH"
 unset STARPU_LAUNCH
+_MS_LAUNCHER="$MS_LAUNCHER"
+unset MS_LAUNCHER
 test_scheds tasks_size_overhead_sched.sh $FAST
