@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2017-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2017-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,11 +17,7 @@
 #include <starpu.h>
 #include "../helper.h"
 
-void free_codelet(void *arg)
-{
-	// The argument of the function is automatically freed by StarPU
-	//	free(arg);
-}
+struct starpu_codelet mycodelet_bis;
 
 void func_cpu_bis(void *descr[], void *_args)
 {
@@ -40,16 +36,14 @@ void func_cpu_bis(void *descr[], void *_args)
 	FPRINTF(stderr, "[msg '%c'] [worker id %d] [worker name %s] [tasks %d]\n", msg, worker_id, worker_name, ntasks);
 	if (ntasks > 0)
 	{
-		struct starpu_codelet *codelet = calloc(1,sizeof(*codelet));
-		codelet->cpu_funcs[0] = func_cpu_bis;
-		codelet->cpu_funcs_name[0] = "func_cpu_bis";
+		int ret;
 		int nntasks = ntasks - 1;
-		starpu_task_insert(codelet,
-				   STARPU_VALUE, &msg, sizeof(msg),
-				   STARPU_VALUE, &nntasks, sizeof(ntasks),
-				   STARPU_VALUE, &worker_id, sizeof(worker_id),
-				   STARPU_CALLBACK_WITH_ARG, free_codelet, codelet,
-				   0);
+		ret = starpu_task_insert(&mycodelet_bis,
+					 STARPU_VALUE, &msg, sizeof(msg),
+					 STARPU_VALUE, &nntasks, sizeof(ntasks),
+					 STARPU_VALUE, &worker_id, sizeof(worker_id),
+					 0);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	}
 }
 
@@ -75,19 +69,23 @@ void func_cpu(void *descr[], void *_args)
 	FPRINTF(stderr, "[msg '%c'] [worker id %d] [worker name %s] [sched_ctx_id %u] [tasks %d] [buffer %p]\n", msg, worker_id, worker_name, sched_ctx_id, ntasks, sched_ctx_id_p);
 	if (ntasks > 0)
 	{
-		struct starpu_codelet *codelet = calloc(1,sizeof(*codelet));
-		codelet->cpu_funcs[0] = func_cpu_bis;
-		codelet->cpu_funcs_name[0] = "func_cpu_bis";
+		int ret;
 		int nntasks = ntasks - 1;
-		starpu_task_insert(codelet,
-				   STARPU_VALUE, &msg, sizeof(msg),
-				   STARPU_VALUE, &nntasks, sizeof(nntasks),
-				   STARPU_VALUE, &worker_id, sizeof(worker_id),
-				   STARPU_CALLBACK_WITH_ARG, free_codelet, codelet,
-				   0);
+		ret = starpu_task_insert(&mycodelet_bis,
+					 STARPU_VALUE, &msg, sizeof(msg),
+					 STARPU_VALUE, &nntasks, sizeof(nntasks),
+					 STARPU_VALUE, &worker_id, sizeof(worker_id),
+					 0);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	}
 }
 
+struct starpu_codelet mycodelet_bis =
+{
+	.cpu_funcs = {func_cpu_bis},
+	.cpu_funcs_name = {"func_cpu_bis"},
+};
+
 struct starpu_codelet mycodelet =
 {
 	.cpu_funcs = {func_cpu},
@@ -96,23 +94,30 @@ struct starpu_codelet mycodelet =
 
 int main(void)
 {
-        int i, ret;
+	int i, ret;
 	int nprocs, nprocs_per_context=1;
-        int procs[STARPU_NMAXWORKERS];
+	int procs[STARPU_NMAXWORKERS];
 	int ntasks=10;
 	char msg[2] = "ab";
 	unsigned *buffer[2];
+	struct starpu_conf conf;
 
-	ret = starpu_init(NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-        nprocs = starpu_cpu_worker_get_count();
+	nprocs = starpu_cpu_worker_get_count();
 	if (nprocs < 2) goto enodev;
 
 	nprocs_per_context = 1;
 	FPRINTF(stderr, "# Workers = %d -> %d worker for each sched context\n", nprocs, nprocs_per_context);
-        starpu_worker_get_ids_by_type(STARPU_CPU_WORKER, procs, nprocs);
+	starpu_worker_get_ids_by_type(STARPU_CPU_WORKER, procs, nprocs);
 
 	unsigned sched_ctx_0 = starpu_sched_ctx_create(procs, nprocs_per_context, "ctx_0", 0);
 	unsigned sched_ctx_1 = starpu_sched_ctx_create(&procs[nprocs_per_context], nprocs_per_context, "ctx_1", 0);
@@ -165,6 +170,6 @@ enodev:
 	starpu_shutdown();
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
- 	 * could perform the kernel, so this is not an error from StarPU */
+	 * could perform the kernel, so this is not an error from StarPU */
 	return STARPU_TEST_SKIPPED;
 }
